# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import model_list_params, model_upload_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.model_list_response import ModelListResponse
from ..types.model_upload_response import ModelUploadResponse

__all__ = ["ModelsResource", "AsyncModelsResource"]


class ModelsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return ModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return ModelsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        dedicated: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelListResponse:
        """
        Lists all of Together's open-source models

        Args:
          dedicated: Filter models to only return dedicated models

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/models",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"dedicated": dedicated}, model_list_params.ModelListParams),
            ),
            cast_to=ModelListResponse,
        )

    def upload(
        self,
        *,
        model_name: str,
        model_source: str,
        base_model: str | Omit = omit,
        description: str | Omit = omit,
        hf_token: str | Omit = omit,
        lora_model: str | Omit = omit,
        model_type: Literal["model", "adapter"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelUploadResponse:
        """
        Upload a custom model or adapter from Hugging Face or S3

        Args:
          model_name: The name to give to your uploaded model

          model_source: The source location of the model (Hugging Face repo or S3 path)

          base_model: The base model to use for an adapter if setting it to run against a serverless
              pool. Only used for model_type `adapter`.

          description: A description of your model

          hf_token: Hugging Face token (if uploading from Hugging Face)

          lora_model: The lora pool to use for an adapter if setting it to run against, say, a
              dedicated pool. Only used for model_type `adapter`.

          model_type: Whether the model is a full model or an adapter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/models",
            body=maybe_transform(
                {
                    "model_name": model_name,
                    "model_source": model_source,
                    "base_model": base_model,
                    "description": description,
                    "hf_token": hf_token,
                    "lora_model": lora_model,
                    "model_type": model_type,
                },
                model_upload_params.ModelUploadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelUploadResponse,
        )


class AsyncModelsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncModelsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        dedicated: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelListResponse:
        """
        Lists all of Together's open-source models

        Args:
          dedicated: Filter models to only return dedicated models

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/models",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"dedicated": dedicated}, model_list_params.ModelListParams),
            ),
            cast_to=ModelListResponse,
        )

    async def upload(
        self,
        *,
        model_name: str,
        model_source: str,
        base_model: str | Omit = omit,
        description: str | Omit = omit,
        hf_token: str | Omit = omit,
        lora_model: str | Omit = omit,
        model_type: Literal["model", "adapter"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ModelUploadResponse:
        """
        Upload a custom model or adapter from Hugging Face or S3

        Args:
          model_name: The name to give to your uploaded model

          model_source: The source location of the model (Hugging Face repo or S3 path)

          base_model: The base model to use for an adapter if setting it to run against a serverless
              pool. Only used for model_type `adapter`.

          description: A description of your model

          hf_token: Hugging Face token (if uploading from Hugging Face)

          lora_model: The lora pool to use for an adapter if setting it to run against, say, a
              dedicated pool. Only used for model_type `adapter`.

          model_type: Whether the model is a full model or an adapter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/models",
            body=await async_maybe_transform(
                {
                    "model_name": model_name,
                    "model_source": model_source,
                    "base_model": base_model,
                    "description": description,
                    "hf_token": hf_token,
                    "lora_model": lora_model,
                    "model_type": model_type,
                },
                model_upload_params.ModelUploadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelUploadResponse,
        )


class ModelsResourceWithRawResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.list = to_raw_response_wrapper(
            models.list,
        )
        self.upload = to_raw_response_wrapper(
            models.upload,
        )


class AsyncModelsResourceWithRawResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.list = async_to_raw_response_wrapper(
            models.list,
        )
        self.upload = async_to_raw_response_wrapper(
            models.upload,
        )


class ModelsResourceWithStreamingResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.list = to_streamed_response_wrapper(
            models.list,
        )
        self.upload = to_streamed_response_wrapper(
            models.upload,
        )


class AsyncModelsResourceWithStreamingResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.list = async_to_streamed_response_wrapper(
            models.list,
        )
        self.upload = async_to_streamed_response_wrapper(
            models.upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .voice_list_response import VoiceListResponse as VoiceListResponse
from .speech_create_params import SpeechCreateParams as SpeechCreateParams
from .translation_create_params import TranslationCreateParams as TranslationCreateParams
from .transcription_create_params import TranscriptionCreateParams as TranscriptionCreateParams
from .translation_create_response import TranslationCreateResponse as TranslationCreateResponse
from .transcription_create_response import TranscriptionCreateResponse as TranscriptionCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["CodeInterpreterExecuteParams", "File"]


class CodeInterpreterExecuteParams(TypedDict, total=False):
    code: Required[str]
    """Code snippet to execute."""

    language: Required[Literal["python"]]
    """Programming language for the code to execute.

    Currently only supports Python, but more will be added.
    """

    files: Iterable[File]
    """Files to upload to the session.

    If present, files will be uploaded before executing the given code.
    """

    session_id: str
    """Identifier of the current session.

    Used to make follow-up calls. Requests will return an error if the session does
    not belong to the caller or has expired.
    """


class File(TypedDict, total=False):
    content: Required[str]

    encoding: Required[Literal["string", "base64"]]
    """Encoding of the file content.

    Use `string` for text files such as code, and `base64` for binary files, such as
    images.
    """

    name: Required[str]



from enum import Enum
from typing import Any, List, Union, Literal, Optional
from datetime import datetime
from typing_extensions import TypeAlias

from pydantic import Field, StrictBool

from ..._models import BaseModel


class FinetuneJobStatus(str, Enum):
    """
    Possible fine-tune job status
    """

    STATUS_PENDING = "pending"
    STATUS_QUEUED = "queued"
    STATUS_RUNNING = "running"
    STATUS_COMPRESSING = "compressing"
    STATUS_UPLOADING = "uploading"
    STATUS_CANCEL_REQUESTED = "cancel_requested"
    STATUS_CANCELLED = "cancelled"
    STATUS_ERROR = "error"
    STATUS_USER_ERROR = "user_error"
    STATUS_COMPLETED = "completed"


class FinetuneEventType(str, Enum):
    """
    Fine-tune job event types
    """

    JOB_PENDING = "JOB_PENDING"
    JOB_START = "JOB_START"
    JOB_STOPPED = "JOB_STOPPED"
    MODEL_DOWNLOADING = "MODEL_DOWNLOADING"
    MODEL_DOWNLOAD_COMPLETE = "MODEL_DOWNLOAD_COMPLETE"
    TRAINING_DATA_DOWNLOADING = "TRAINING_DATA_DOWNLOADING"
    TRAINING_DATA_DOWNLOAD_COMPLETE = "TRAINING_DATA_DOWNLOAD_COMPLETE"
    VALIDATION_DATA_DOWNLOADING = "VALIDATION_DATA_DOWNLOADING"
    VALIDATION_DATA_DOWNLOAD_COMPLETE = "VALIDATION_DATA_DOWNLOAD_COMPLETE"
    WANDB_INIT = "WANDB_INIT"
    TRAINING_START = "TRAINING_START"
    CHECKPOINT_SAVE = "CHECKPOINT_SAVE"
    BILLING_LIMIT = "BILLING_LIMIT"
    EPOCH_COMPLETE = "EPOCH_COMPLETE"
    EVAL_COMPLETE = "EVAL_COMPLETE"
    TRAINING_COMPLETE = "TRAINING_COMPLETE"
    MODEL_COMPRESSING = "COMPRESSING_MODEL"
    MODEL_COMPRESSION_COMPLETE = "MODEL_COMPRESSION_COMPLETE"
    MODEL_UPLOADING = "MODEL_UPLOADING"
    MODEL_UPLOAD_COMPLETE = "MODEL_UPLOAD_COMPLETE"
    MODEL_UPLOADING_TO_HF = "MODEL_UPLOADING_TO_HF"
    MODEL_UPLOAD_TO_HF_COMPLETE = "MODEL_UPLOAD_TO_HF_COMPLETE"
    JOB_COMPLETE = "JOB_COMPLETE"
    JOB_ERROR = "JOB_ERROR"
    JOB_USER_ERROR = "JOB_USER_ERROR"
    CANCEL_REQUESTED = "CANCEL_REQUESTED"
    JOB_RESTARTED = "JOB_RESTARTED"
    REFUND = "REFUND"
    WARNING = "WARNING"


class FinetuneEventLevels(str, Enum):
    """
    Fine-tune job event status levels
    """

    NULL = ""
    INFO = "Info"
    WARNING = "Warning"
    ERROR = "Error"


class FinetuneEvent(BaseModel):
    """
    Fine-tune event type
    """

    # object type
    object: Literal["fine-tune-event"]
    # created at datetime stamp
    created_at: Union[str, None] = None
    # event log level
    level: Union[FinetuneEventLevels, str, None] = None
    # event message string
    message: Union[str, None] = None
    # event type
    type: Union[FinetuneEventType, None] = None
    # optional: model parameter count
    param_count: Union[int, None] = None
    # optional: dataset token count
    token_count: Union[int, None] = None
    # optional: weights & biases url
    wandb_url: Union[str, None] = None
    # event hash
    hash: Union[str, None] = None


class FullTrainingType(BaseModel):
    """
    Training type for full fine-tuning
    """

    type: Union[Literal["Full"], Literal[""]] = "Full"


class LoRATrainingType(BaseModel):
    """
    Training type for LoRA adapters training
    """

    lora_r: int
    lora_alpha: int
    lora_dropout: float = 0.0
    lora_trainable_modules: str = "all-linear"
    type: Literal["Lora"] = "Lora"


class UnknownTrainingType(BaseModel):
    """
    Catch-all for unknown training types (forward compatibility).
    Accepts any training type not explicitly defined.
    """

    type: str


TrainingType: TypeAlias = Union[
    FullTrainingType,
    LoRATrainingType,
    UnknownTrainingType,
]


class FinetuneFullTrainingLimits(BaseModel):
    max_batch_size: int
    max_batch_size_dpo: int = -1
    min_batch_size: int

    def __init__(self, **data: Any) -> None:
        super().__init__(**data)
        if self.max_batch_size_dpo == -1:
            half_max = self.max_batch_size // 2
            rounded_half_max = (half_max // 8) * 8
            self.max_batch_size_dpo = max(self.min_batch_size, rounded_half_max)


class FinetuneLoraTrainingLimits(FinetuneFullTrainingLimits):
    max_rank: int
    target_modules: List[str]


class TrainingMethodSFT(BaseModel):
    """
    Training method type for SFT training
    """

    method: Literal["sft"] = "sft"
    train_on_inputs: Union[StrictBool, Literal["auto"]] = "auto"


class TrainingMethodDPO(BaseModel):
    """
    Training method type for DPO training
    """

    method: Literal["dpo"] = "dpo"
    dpo_beta: Union[float, None] = None
    dpo_normalize_logratios_by_length: bool = False
    dpo_reference_free: bool = False
    rpo_alpha: Union[float, None] = None
    simpo_gamma: Union[float, None] = None


class TrainingMethodUnknown(BaseModel):
    """
    Catch-all for unknown training methods (forward compatibility).
    Accepts any training method not explicitly defined.
    """

    method: str

TrainingMethod: TypeAlias = Union[
    TrainingMethodSFT,
    TrainingMethodDPO,
    TrainingMethodUnknown,
]


class FinetuneTrainingLimits(BaseModel):
    max_num_epochs: int
    max_learning_rate: float
    min_learning_rate: float
    full_training: Optional[FinetuneFullTrainingLimits] = None
    lora_training: Optional[FinetuneLoraTrainingLimits] = None


class LinearLRSchedulerArgs(BaseModel):
    """
    Linear learning rate scheduler arguments
    """

    min_lr_ratio: Union[float, None] = 0.0


class CosineLRSchedulerArgs(BaseModel):
    """
    Cosine learning rate scheduler arguments
    """

    min_lr_ratio: Union[float, None] = 0.0
    num_cycles: Union[float, None] = 0.5


class LinearLRScheduler(BaseModel):
    """
    Linear learning rate scheduler
    """

    lr_scheduler_type: Literal["linear"] = "linear"
    lr_scheduler_args: Union[LinearLRSchedulerArgs, None] = None


class CosineLRScheduler(BaseModel):
    """
    Cosine learning rate scheduler
    """

    lr_scheduler_type: Literal["cosine"] = "cosine"
    lr_scheduler_args: Union[CosineLRSchedulerArgs, None] = None


class EmptyLRScheduler(BaseModel):
    """
    Empty learning rate scheduler

    Placeholder for old fine-tuning jobs with no lr_scheduler_type specified
    """

    lr_scheduler_type: Literal[""]
    lr_scheduler_args: None = None

class UnknownLRScheduler(BaseModel):
    """
    Unknown learning rate scheduler

    Catch-all for unknown LR scheduler types (forward compatibility)
    """

    lr_scheduler_type: str
    lr_scheduler_args: Optional[Any] = None


FinetuneLRScheduler: TypeAlias = Union[
    LinearLRScheduler,
    CosineLRScheduler,
    EmptyLRScheduler,
    UnknownLRScheduler,
]


class FinetuneResponse(BaseModel):
    """
    Fine-tune API response type
    """

    id: str
    """Unique identifier for the fine-tune job"""

    created_at: datetime
    """Creation timestamp of the fine-tune job"""

    status: Optional[Union[FinetuneJobStatus, str]] = None
    """Status of the fine-tune job (accepts known enum values or string for forward compatibility)"""

    updated_at: datetime
    """Last update timestamp of the fine-tune job"""

    batch_size: Optional[int] = None
    """Batch size used for training"""

    events: Optional[List[Union[FinetuneEvent, str]]] = None
    """Events related to this fine-tune job (accepts known enum values or string for forward compatibility)"""

    from_checkpoint: Optional[str] = None
    """Checkpoint used to continue training"""

    from_hf_model: Optional[str] = None
    """Hugging Face Hub repo to start training from"""

    hf_model_revision: Optional[str] = None
    """The revision of the Hugging Face Hub model to continue training from"""

    learning_rate: Optional[float] = None
    """Learning rate used for training"""

    lr_scheduler: Optional[FinetuneLRScheduler] = None
    """Learning rate scheduler configuration"""

    max_grad_norm: Optional[float] = None
    """Maximum gradient norm for clipping"""

    model: Optional[str] = None
    """Base model used for fine-tuning"""

    output_name: Optional[str] = Field(alias="model_output_name")
    """Output model name"""

    adapter_output_name: Optional[str]
    """Adapter output name"""

    n_checkpoints: Optional[int] = None
    """Number of checkpoints saved during training"""

    n_epochs: Optional[int] = None
    """Number of training epochs"""

    n_evals: Optional[int] = None
    """Number of evaluations during training"""

    owner_address: Optional[str] = None
    """Owner address information"""

    suffix: Optional[str] = None
    """Suffix added to the fine-tuned model name"""

    token_count: Optional[int] = None
    """Count of tokens processed"""

    total_price: Optional[int] = None
    """Total price for the fine-tuning job"""

    training_file: Optional[str] = None
    """File-ID of the training file"""

    training_method: Optional[TrainingMethod] = None
    """Method of training used"""

    training_type: Optional[TrainingType] = None
    """Type of training used (full or LoRA)"""

    user_id: Optional[str] = None
    """Identifier for the user who created the job"""

    validation_file: Optional[str] = None
    """File-ID of the validation file"""

    wandb_name: Optional[str] = None
    """Weights & Biases run name"""

    wandb_project_name: Optional[str] = None
    """Weights & Biases project name"""

    wandb_base_url: Union[str, None] = None
    """Weights & Biases base URL"""

    wandb_url: Union[str, None] = None
    """Weights & Biases job URL"""

    warmup_ratio: Optional[float] = None
    """Ratio of warmup steps"""

    weight_decay: Optional[float] = None
    """Weight decay value used"""

    eval_steps: Union[int, None] = None
    """number of steps between evals"""

    job_id: Optional[str] = None
    """Job ID"""

    param_count: Optional[int] = None
    """Model parameter count"""

    total_steps: Optional[int] = None
    """Total number of training steps"""

    steps_completed: Union[int, None] = None
    """Number of steps completed (incrementing counter)"""

    epochs_completed: Union[int, None] = None
    """Number of epochs completed (incrementing counter)"""

    evals_completed: Union[int, None] = None
    """Number of evaluation loops completed (incrementing counter)"""

    queue_depth: Union[int, None] = None
    """Place in job queue (decrementing counter)"""

    # # training file metadata
    training_file_num_lines: Optional[int] = Field(None, alias="TrainingFileNumLines")
    training_file_size: Optional[int] = Field(None, alias="TrainingFileSize")
    train_on_inputs: Union[StrictBool, Literal["auto"], None] = "auto"

    @classmethod
    def validate_training_type(cls, v: TrainingType) -> TrainingType:
        if v.type == "Full" or v.type == "":
            return FullTrainingType(**v.model_dump())
        elif v.type == "Lora":
            return LoRATrainingType(**v.model_dump())
        else:
            raise ValueError("Unknown training type")


class FinetuneRequest(BaseModel):
    """
    Fine-tune request type
    """

    # training file ID
    training_file: str
    # validation file id
    validation_file: Union[str, None] = None
    # base model string
    model: Union[str, None] = None
    # number of epochs to train for
    n_epochs: int
    # training learning rate
    learning_rate: float
    # learning rate scheduler type and args
    lr_scheduler: Union[FinetuneLRScheduler, None] = None
    # learning rate warmup ratio
    warmup_ratio: float
    # max gradient norm
    max_grad_norm: float
    # weight decay
    weight_decay: float
    # number of checkpoints to save
    n_checkpoints: Union[int, None] = None
    # number of evaluation loops to run
    n_evals: Union[int, None] = None
    # training batch size
    batch_size: Union[int, Literal["max"], None] = None
    # up to 40 character suffix for output model name
    suffix: Union[str, None] = None
    # weights & biases api key
    wandb_key: Union[str, None] = None
    # weights & biases base url
    wandb_base_url: Union[str, None] = None
    # wandb project name
    wandb_project_name: Union[str, None] = None
    # wandb run name
    wandb_name: Union[str, None] = None
    # training type
    training_type: Union[TrainingType, None] = None
    # training method
    training_method: TrainingMethod = Field(default_factory=TrainingMethodSFT)
    # from step
    from_checkpoint: Union[str, None] = None
    from_hf_model: Union[str, None] = None
    hf_model_revision: Union[str, None] = None
    # hf related fields
    hf_api_token: Union[str, None] = None
    hf_output_repo_name: Union[str, None] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "FineTuningEstimatePriceParams",
    "TrainingMethod",
    "TrainingMethodTrainingMethodSft",
    "TrainingMethodTrainingMethodDpo",
    "TrainingType",
    "TrainingTypeFullTrainingType",
    "TrainingTypeLoRaTrainingType",
]


class FineTuningEstimatePriceParams(TypedDict, total=False):
    training_file: Required[str]
    """File-ID of a training file uploaded to the Together API"""

    from_checkpoint: str
    """The checkpoint identifier to continue training from a previous fine-tuning job.

    Format is `{$JOB_ID}` or `{$OUTPUT_MODEL_NAME}` or `{$JOB_ID}:{$STEP}` or
    `{$OUTPUT_MODEL_NAME}:{$STEP}`. The step value is optional; without it, the
    final checkpoint will be used.
    """

    model: str
    """Name of the base model to run fine-tune job on"""

    n_epochs: int
    """
    Number of complete passes through the training dataset (higher values may
    improve results but increase cost and risk of overfitting)
    """

    n_evals: int
    """Number of evaluations to be run on a given validation set during training"""

    training_method: TrainingMethod
    """The training method to use.

    'sft' for Supervised Fine-Tuning or 'dpo' for Direct Preference Optimization.
    """

    training_type: TrainingType

    validation_file: str
    """File-ID of a validation file uploaded to the Together API"""


class TrainingMethodTrainingMethodSft(TypedDict, total=False):
    method: Required[Literal["sft"]]

    train_on_inputs: Required[Union[bool, Literal["auto"]]]
    """
    Whether to mask the user messages in conversational data or prompts in
    instruction data.
    """


class TrainingMethodTrainingMethodDpo(TypedDict, total=False):
    method: Required[Literal["dpo"]]

    dpo_beta: float

    dpo_normalize_logratios_by_length: bool

    dpo_reference_free: bool

    rpo_alpha: float

    simpo_gamma: float


TrainingMethod: TypeAlias = Union[TrainingMethodTrainingMethodSft, TrainingMethodTrainingMethodDpo]


class TrainingTypeFullTrainingType(TypedDict, total=False):
    type: Required[Literal["Full"]]


class TrainingTypeLoRaTrainingType(TypedDict, total=False):
    lora_alpha: Required[int]

    lora_r: Required[int]

    type: Required[Literal["Lora"]]

    lora_dropout: float

    lora_trainable_modules: str


TrainingType: TypeAlias = Union[TrainingTypeFullTrainingType, TrainingTypeLoRaTrainingType]



import csv
import json
from typing import Any, Dict, List
from pathlib import Path

from together.lib.utils.files import check_file


def test_check_jsonl_valid_general(tmp_path: Path):
    # Create a valid JSONL file
    file = tmp_path / "valid.jsonl"
    content = [{"text": "Hello, world!"}, {"text": "How are you?"}]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert report["is_check_passed"]
    assert report["utf8"]
    assert report["num_samples"] == len(content)
    assert report["has_min_samples"]


def test_check_jsonl_valid_instruction(tmp_path: Path):
    # Create a valid JSONL file with instruction format
    file = tmp_path / "valid_instruction.jsonl"
    content = [
        {"prompt": "Translate the following sentence.", "completion": "Hello, world!"},
        {
            "prompt": "Summarize the text.",
            "completion": "Weyland-Yutani Corporation creates advanced AI.",
        },
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert report["is_check_passed"]
    assert report["utf8"]
    assert report["num_samples"] == len(content)
    assert report["has_min_samples"]


def test_check_jsonl_valid_conversational_single_turn(tmp_path: Path):
    # Create a valid JSONL file with conversational format and 1 user-assistant turn pair
    file = tmp_path / "valid_conversational_single_turn.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Hello"},
                {"role": "assistant", "content": "Hi there!"},
            ]
        },
        {
            "messages": [
                {"role": "user", "content": "How are you?"},
                {"role": "assistant", "content": "I am fine."},
            ]
        },
        {
            "messages": [
                {"role": "system", "content": "You are a kind AI"},
                {"role": "user", "content": "How are you?"},
                {"role": "assistant", "content": "I am fine."},
            ]
        },
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert report["is_check_passed"]
    assert report["utf8"]
    assert report["num_samples"] == len(content)
    assert report["has_min_samples"]


def test_check_jsonl_valid_conversational_multiple_turns(tmp_path: Path):
    # Create a valid JSONL file with conversational format and multiple user-assistant turn pairs
    file = tmp_path / "valid_conversational_multiple_turns.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Is it going to rain today?"},
                {
                    "role": "assistant",
                    "content": "Yes, expect showers in the afternoon.",
                },
                {"role": "user", "content": "What is the weather like in Tokyo?"},
                {"role": "assistant", "content": "It is sunny with a chance of rain."},
            ]
        },
        {
            "messages": [
                {"role": "user", "content": "Who won the game last night?"},
                {"role": "assistant", "content": "The home team won by two points."},
                {"role": "user", "content": "What is the weather like in Amsterdam?"},
                {"role": "assistant", "content": "It is cloudy with a chance of snow."},
            ]
        },
        {
            "messages": [
                {"role": "system", "content": "You are a kind AI"},
                {"role": "user", "content": "Who won the game last night?"},
                {"role": "assistant", "content": "The home team won by two points."},
                {"role": "user", "content": "What is the weather like in Amsterdam?"},
                {"role": "assistant", "content": "It is cloudy with a chance of snow."},
            ]
        },
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert report["is_check_passed"]
    assert report["utf8"]
    assert report["num_samples"] == len(content)
    assert report["has_min_samples"]


def test_check_jsonl_empty_file(tmp_path: Path):
    # Create an empty JSONL file
    file = tmp_path / "empty.jsonl"
    file.touch()

    report = check_file(file)

    assert not report["is_check_passed"]
    assert report["message"] == "File is empty"
    assert report["file_size"] == 0


def test_check_jsonl_non_utf8(tmp_path: Path):
    # Create a non-UTF-8 encoded JSONL file
    file = tmp_path / "non_utf8.jsonl"
    file.write_bytes(b"\xff\xfe\xfd")

    report = check_file(file)

    assert not report["is_check_passed"]
    assert not report["utf8"]
    assert "File is not UTF-8 encoded." in report["message"]


def test_check_jsonl_invalid_json(tmp_path: Path):
    # Create a JSONL file with invalid JSON
    file = tmp_path / "invalid_json.jsonl"
    content = [{"text": "Hello, world!"}, "Invalid JSON Line"]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert not report["is_check_passed"]
    assert "Error parsing file." in report["message"]


def test_check_jsonl_missing_required_field(tmp_path: Path):
    # Create a JSONL file missing a required field
    file = tmp_path / "missing_field.jsonl"
    content = [
        {"prompt": "Translate the following sentence.", "completion": "Hello, world!"},
        {"prompt": "Summarize the text."},
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert not report["is_check_passed"]
    assert "Error parsing file. Could not detect a format for the line 2" in report["message"]


def test_check_jsonl_inconsistent_dataset_format(tmp_path: Path):
    # Create a JSONL file with inconsistent dataset formats
    file = tmp_path / "inconsistent_format.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Hi"},
                {"role": "assistant", "content": "Hi! How can I help you?"},
            ]
        },
        {"text": "How are you?"},  # Missing 'messages'
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert not report["is_check_passed"]
    assert "All samples in the dataset must have the same dataset format" in report["message"]


def test_check_jsonl_invalid_role(tmp_path: Path):
    # Create a JSONL file with an invalid role
    file = tmp_path / "invalid_role.jsonl"
    content = [{"messages": [{"role": "invalid_role", "content": "Hi"}]}]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert not report["is_check_passed"]
    assert "Invalid role `invalid_role` in conversation" in report["message"]


def test_check_jsonl_non_alternating_roles(tmp_path: Path):
    # Create a JSONL file with non-alternating user/assistant roles
    file = tmp_path / "non_alternating_roles.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Hi"},
                {"role": "user", "content": "Hello again"},
            ]
        }
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert not report["is_check_passed"]
    assert "Invalid role turns" in report["message"]


def test_check_jsonl_assistant_role_exists(tmp_path: Path):
    # Create a JSONL file with no assistant role
    file = tmp_path / "assistant_role_exists.jsonl"
    content = [{"messages": [{"role": "user", "content": "Hi"}]}]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)

    assert not report["is_check_passed"]
    assert "At least one message with the assistant role must be present" in report["message"]


def test_check_jsonl_invalid_value_type(tmp_path: Path):
    # Create a JSONL file with an invalid value type
    file = tmp_path / "invalid_value_type.jsonl"
    content = [{"text": 123}]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert not report["is_check_passed"]
    assert "Expected string" in report["message"]


def test_check_jsonl_missing_field_in_conversation(tmp_path: Path):
    file = tmp_path / "missing_field_in_conversation.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Hi"},
                {"role": "assistant"},
            ]
        }
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert not report["is_check_passed"]
    assert "Missing required column `content`" in report["message"]


def test_check_jsonl_wrong_turn_type(tmp_path: Path):
    file = tmp_path / "wrong_turn_type.jsonl"
    content = [
        {
            "messages": [
                "Hi!",
                {"role": "user", "content": "Hi"},
                {"role": "assistant"},
            ]
        }
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert not report["is_check_passed"]
    assert (
        "Invalid format on line 1 of the input file. The `messages` column must be a list of dicts."
        in report["message"]
    )


def test_check_jsonl_extra_column(tmp_path: Path):
    file = tmp_path / "extra_column.jsonl"
    content = [{"text": "Hello, world!", "extra_column": "extra"}]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert not report["is_check_passed"]
    assert "Found extra column" in report["message"]


def test_check_jsonl_empty_messages(tmp_path: Path):
    file = tmp_path / "empty_messages.jsonl"
    content: List[Dict[str, Any]] = [{"messages": []}]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert not report["is_check_passed"]
    assert "The `messages` column must not be empty" in report["message"]


def test_check_jsonl_valid_weights_all_messages(tmp_path: Path):
    file = tmp_path / "valid_weights_all.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Hello", "weight": 1},
                {"role": "assistant", "content": "Hi there!", "weight": 0},
                {"role": "user", "content": "How are you?", "weight": 1},
                {"role": "assistant", "content": "I'm doing well!", "weight": 1},
            ]
        },
        {
            "messages": [
                {"role": "system", "content": "You are helpful", "weight": 0},
                {"role": "user", "content": "What's the weather?", "weight": 1},
                {"role": "assistant", "content": "It's sunny today!", "weight": 1},
            ]
        },
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert report["is_check_passed"]
    assert report["num_samples"] == len(content)


def test_check_jsonl_valid_weights_mixed_with_none(tmp_path: Path):
    file = tmp_path / "valid_weights_mixed.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Hello", "weight": 1},
                {"role": "assistant", "content": "Hi there!", "weight": 0},
                {"role": "user", "content": "How are you?"},
                {"role": "assistant", "content": "I'm doing well!"},
            ]
        },
        {
            "messages": [
                {"role": "user", "content": "What's the weather?"},
                {"role": "assistant", "content": "It's sunny today!"},
            ]
        },
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert report["is_check_passed"]
    assert report["num_samples"] == len(content)


def test_check_jsonl_invalid_weight_float(tmp_path: Path):
    file = tmp_path / "invalid_weight_float.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Hello", "weight": 1.0},
                {"role": "assistant", "content": "Hi there!", "weight": 0},
            ]
        }
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert not report["is_check_passed"]
    assert "Weight must be an integer" in report["message"]


def test_check_jsonl_invalid_weight(tmp_path: Path):
    file = tmp_path / "invalid_weight.jsonl"
    content = [
        {
            "messages": [
                {"role": "user", "content": "Hello", "weight": 2},
                {"role": "assistant", "content": "Hi there!", "weight": 0},
            ]
        }
    ]
    with file.open("w") as f:
        f.write("\n".join(json.dumps(item) for item in content))

    report = check_file(file)
    assert not report["is_check_passed"]
    assert "Weight must be either 0 or 1" in report["message"]


def test_check_csv_valid_general(tmp_path: Path):
    # Create a valid CSV file
    file = tmp_path / "valid.csv"
    with open(file, "w") as f:
        writer = csv.DictWriter(f, fieldnames=["text"])
        writer.writeheader()
        writer.writerow({"text": "Hello, world!"})
        writer.writerow({"text": "How are you?"})

    report = check_file(file, purpose="eval")
    assert report["is_check_passed"]
    assert report["utf8"]
    assert report["num_samples"] == 2
    assert report["has_min_samples"]


def test_check_csv_empty_file(tmp_path: Path):
    # Create an empty CSV file
    file = tmp_path / "empty.csv"
    file.touch()

    report = check_file(file, purpose="eval")

    assert not report["is_check_passed"]
    assert report["message"] == "File is empty"
    assert report["file_size"] == 0


def test_check_csv_valid_completion(tmp_path: Path):
    # Create a valid CSV file with conversational format
    file = tmp_path / "valid_completion.csv"

    with open(file, "w") as f:
        writer = csv.DictWriter(f, fieldnames=["prompt", "completion"])
        writer.writeheader()
        writer.writerow(
            {
                "prompt": "Translate the following sentence.",
                "completion": "Hello, world!",
            }
        )

    report = check_file(file, purpose="eval")
    assert report["is_check_passed"]
    assert report["utf8"]
    assert report["num_samples"] == 1
    assert report["has_min_samples"]


def test_check_csv_invalid_column(tmp_path: Path):
    # Create a CSV file with an invalid column
    file = tmp_path / "invalid_column.csv"
    with open(file, "w") as f:
        writer = csv.DictWriter(f, fieldnames=["asfg"])
        writer.writeheader()

    report = check_file(file)

    assert not report["is_check_passed"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["FineTuningListCheckpointsResponse", "Data"]


class Data(BaseModel):
    checkpoint_type: str

    created_at: str

    path: str

    step: int


class FineTuningListCheckpointsResponse(BaseModel):
    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .image_data_b64 import ImageDataB64
from .image_data_url import ImageDataURL

__all__ = ["ImageFile", "Data"]

Data: TypeAlias = Annotated[Union[ImageDataB64, ImageDataURL], PropertyInfo(discriminator="type")]


class ImageFile(BaseModel):
    id: str

    data: List[Data]

    model: str

    object: Literal["list"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["ImageGenerateParams", "ImageLora"]


class ImageGenerateParams(TypedDict, total=False):
    model: Required[
        Union[
            Literal[
                "black-forest-labs/FLUX.1-schnell-Free",
                "black-forest-labs/FLUX.1-schnell",
                "black-forest-labs/FLUX.1.1-pro",
            ],
            str,
        ]
    ]
    """The model to use for image generation.

    [See all of Together AI's image models](https://docs.together.ai/docs/serverless-models#image-models)
    """

    prompt: Required[str]
    """A description of the desired images. Maximum length varies by model."""

    disable_safety_checker: bool
    """If true, disables the safety checker for image generation."""

    guidance_scale: float
    """Adjusts the alignment of the generated image with the input prompt.

    Higher values (e.g., 8-10) make the output more faithful to the prompt, while
    lower values (e.g., 1-5) encourage more creative freedom.
    """

    height: int
    """Height of the image to generate in number of pixels."""

    image_loras: Iterable[ImageLora]
    """
    An array of objects that define LoRAs (Low-Rank Adaptations) to influence the
    generated image.
    """

    image_url: str
    """URL of an image to use for image models that support it."""

    n: int
    """Number of image results to generate."""

    negative_prompt: str
    """The prompt or prompts not to guide the image generation."""

    output_format: Literal["jpeg", "png"]
    """The format of the image response.

    Can be either be `jpeg` or `png`. Defaults to `jpeg`.
    """

    reference_images: SequenceNotStr[str]
    """
    An array of image URLs that guide the overall appearance and style of the
    generated image. These reference images influence the visual characteristics
    consistently across the generation.
    """

    response_format: Literal["base64", "url"]
    """Format of the image response. Can be either a base64 string or a URL."""

    seed: int
    """Seed used for generation. Can be used to reproduce image generations."""

    steps: int
    """Number of generation steps."""

    width: int
    """Width of the image to generate in number of pixels."""


class ImageLora(TypedDict, total=False):
    path: Required[str]
    """The URL of the LoRA to apply (e.g.

    https://huggingface.co/strangerzonehf/Flux-Midjourney-Mix2-LoRA).
    """

    scale: Required[float]
    """The strength of the LoRA's influence. Most LoRA's recommend a value of 1."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["HardwareListResponse", "Data", "DataPricing", "DataSpecs", "DataAvailability"]


class DataPricing(BaseModel):
    """Pricing details for using an endpoint"""

    cents_per_minute: float
    """Cost per minute of endpoint uptime in cents"""


class DataSpecs(BaseModel):
    """Detailed specifications of a hardware configuration"""

    gpu_count: int
    """Number of GPUs in this configuration"""

    gpu_link: str
    """The GPU interconnect technology"""

    gpu_memory: float
    """Amount of GPU memory in GB"""

    gpu_type: str
    """The type/model of GPU"""


class DataAvailability(BaseModel):
    """Indicates the current availability status of a hardware configuration"""

    status: Literal["available", "unavailable", "insufficient"]
    """The availability status of the hardware configuration"""


class Data(BaseModel):
    """Hardware configuration details with optional availability status"""

    id: str
    """Unique identifier for the hardware configuration"""

    object: Literal["hardware"]

    pricing: DataPricing
    """Pricing details for using an endpoint"""

    specs: DataSpecs
    """Detailed specifications of a hardware configuration"""

    updated_at: datetime
    """Timestamp of when the hardware status was last updated"""

    availability: Optional[DataAvailability] = None
    """Indicates the current availability status of a hardware configuration"""


class HardwareListResponse(BaseModel):
    data: List[Data]

    object: Literal["list"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Embedding", "Data"]


class Data(BaseModel):
    embedding: List[float]

    index: int

    object: Literal["embedding"]


class Embedding(BaseModel):
    data: List[Data]

    model: str

    object: Literal["list"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .model_object import ModelObject

__all__ = ["ModelListResponse"]

ModelListResponse: TypeAlias = List[ModelObject]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Iterable, cast
from typing_extensions import Literal

import httpx

from ...types import code_interpreter_execute_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .sessions import (
    SessionsResource,
    AsyncSessionsResource,
    SessionsResourceWithRawResponse,
    AsyncSessionsResourceWithRawResponse,
    SessionsResourceWithStreamingResponse,
    AsyncSessionsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.execute_response import ExecuteResponse

__all__ = ["CodeInterpreterResource", "AsyncCodeInterpreterResource"]


class CodeInterpreterResource(SyncAPIResource):
    @cached_property
    def sessions(self) -> SessionsResource:
        return SessionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CodeInterpreterResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return CodeInterpreterResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CodeInterpreterResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return CodeInterpreterResourceWithStreamingResponse(self)

    def execute(
        self,
        *,
        code: str,
        language: Literal["python"],
        files: Iterable[code_interpreter_execute_params.File] | Omit = omit,
        session_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExecuteResponse:
        """Executes the given code snippet and returns the output.

        Without a session_id, a
        new session will be created to run the code. If you do pass in a valid
        session_id, the code will be run in that session. This is useful for running
        multiple code snippets in the same environment, because dependencies and similar
        things are persisted between calls to the same session.

        Args:
          code: Code snippet to execute.

          language: Programming language for the code to execute. Currently only supports Python,
              but more will be added.

          files: Files to upload to the session. If present, files will be uploaded before
              executing the given code.

          session_id: Identifier of the current session. Used to make follow-up calls. Requests will
              return an error if the session does not belong to the caller or has expired.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            ExecuteResponse,
            self._post(
                "/tci/execute",
                body=maybe_transform(
                    {
                        "code": code,
                        "language": language,
                        "files": files,
                        "session_id": session_id,
                    },
                    code_interpreter_execute_params.CodeInterpreterExecuteParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ExecuteResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncCodeInterpreterResource(AsyncAPIResource):
    @cached_property
    def sessions(self) -> AsyncSessionsResource:
        return AsyncSessionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCodeInterpreterResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncCodeInterpreterResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCodeInterpreterResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncCodeInterpreterResourceWithStreamingResponse(self)

    async def execute(
        self,
        *,
        code: str,
        language: Literal["python"],
        files: Iterable[code_interpreter_execute_params.File] | Omit = omit,
        session_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExecuteResponse:
        """Executes the given code snippet and returns the output.

        Without a session_id, a
        new session will be created to run the code. If you do pass in a valid
        session_id, the code will be run in that session. This is useful for running
        multiple code snippets in the same environment, because dependencies and similar
        things are persisted between calls to the same session.

        Args:
          code: Code snippet to execute.

          language: Programming language for the code to execute. Currently only supports Python,
              but more will be added.

          files: Files to upload to the session. If present, files will be uploaded before
              executing the given code.

          session_id: Identifier of the current session. Used to make follow-up calls. Requests will
              return an error if the session does not belong to the caller or has expired.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            ExecuteResponse,
            await self._post(
                "/tci/execute",
                body=await async_maybe_transform(
                    {
                        "code": code,
                        "language": language,
                        "files": files,
                        "session_id": session_id,
                    },
                    code_interpreter_execute_params.CodeInterpreterExecuteParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ExecuteResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class CodeInterpreterResourceWithRawResponse:
    def __init__(self, code_interpreter: CodeInterpreterResource) -> None:
        self._code_interpreter = code_interpreter

        self.execute = to_raw_response_wrapper(
            code_interpreter.execute,
        )

    @cached_property
    def sessions(self) -> SessionsResourceWithRawResponse:
        return SessionsResourceWithRawResponse(self._code_interpreter.sessions)


class AsyncCodeInterpreterResourceWithRawResponse:
    def __init__(self, code_interpreter: AsyncCodeInterpreterResource) -> None:
        self._code_interpreter = code_interpreter

        self.execute = async_to_raw_response_wrapper(
            code_interpreter.execute,
        )

    @cached_property
    def sessions(self) -> AsyncSessionsResourceWithRawResponse:
        return AsyncSessionsResourceWithRawResponse(self._code_interpreter.sessions)


class CodeInterpreterResourceWithStreamingResponse:
    def __init__(self, code_interpreter: CodeInterpreterResource) -> None:
        self._code_interpreter = code_interpreter

        self.execute = to_streamed_response_wrapper(
            code_interpreter.execute,
        )

    @cached_property
    def sessions(self) -> SessionsResourceWithStreamingResponse:
        return SessionsResourceWithStreamingResponse(self._code_interpreter.sessions)


class AsyncCodeInterpreterResourceWithStreamingResponse:
    def __init__(self, code_interpreter: AsyncCodeInterpreterResource) -> None:
        self._code_interpreter = code_interpreter

        self.execute = async_to_streamed_response_wrapper(
            code_interpreter.execute,
        )

    @cached_property
    def sessions(self) -> AsyncSessionsResourceWithStreamingResponse:
        return AsyncSessionsResourceWithStreamingResponse(self._code_interpreter.sessions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .batch_job import BatchJob as BatchJob
from .embedding import Embedding as Embedding
from .file_list import FileList as FileList
from .file_type import FileType as FileType
from .log_probs import LogProbs as LogProbs
from .video_job import VideoJob as VideoJob
from .completion import Completion as Completion
from .image_file import ImageFile as ImageFile
from .autoscaling import Autoscaling as Autoscaling
from .tool_choice import ToolChoice as ToolChoice
from .tools_param import ToolsParam as ToolsParam
from .file_purpose import FilePurpose as FilePurpose
from .model_object import ModelObject as ModelObject
from .file_response import FileResponse as FileResponse
from .evaluation_job import EvaluationJob as EvaluationJob
from .finetune_event import FinetuneEvent as FinetuneEvent
from .image_data_b64 import ImageDataB64 as ImageDataB64
from .image_data_url import ImageDataURL as ImageDataURL
from .completion_chunk import CompletionChunk as CompletionChunk
from .eval_list_params import EvalListParams as EvalListParams
from .execute_response import ExecuteResponse as ExecuteResponse
from .autoscaling_param import AutoscalingParam as AutoscalingParam
from .finetune_response import FinetuneResponse as FinetuneResponse
from .job_list_response import JobListResponse as JobListResponse
from .model_list_params import ModelListParams as ModelListParams
from .tool_choice_param import ToolChoiceParam as ToolChoiceParam
from .dedicated_endpoint import DedicatedEndpoint as DedicatedEndpoint
from .eval_create_params import EvalCreateParams as EvalCreateParams
from .eval_list_response import EvalListResponse as EvalListResponse
from .batch_create_params import BatchCreateParams as BatchCreateParams
from .batch_list_response import BatchListResponse as BatchListResponse
from .finetune_event_type import FinetuneEventType as FinetuneEventType
from .model_list_response import ModelListResponse as ModelListResponse
from .model_upload_params import ModelUploadParams as ModelUploadParams
from .video_create_params import VideoCreateParams as VideoCreateParams
from .endpoint_list_params import EndpointListParams as EndpointListParams
from .eval_create_response import EvalCreateResponse as EvalCreateResponse
from .eval_status_response import EvalStatusResponse as EvalStatusResponse
from .file_delete_response import FileDeleteResponse as FileDeleteResponse
from .hardware_list_params import HardwareListParams as HardwareListParams
from .rerank_create_params import RerankCreateParams as RerankCreateParams
from .batch_create_response import BatchCreateResponse as BatchCreateResponse
from .image_generate_params import ImageGenerateParams as ImageGenerateParams
from .job_retrieve_response import JobRetrieveResponse as JobRetrieveResponse
from .model_upload_response import ModelUploadResponse as ModelUploadResponse
from .endpoint_create_params import EndpointCreateParams as EndpointCreateParams
from .endpoint_list_response import EndpointListResponse as EndpointListResponse
from .endpoint_update_params import EndpointUpdateParams as EndpointUpdateParams
from .hardware_list_response import HardwareListResponse as HardwareListResponse
from .rerank_create_response import RerankCreateResponse as RerankCreateResponse
from .embedding_create_params import EmbeddingCreateParams as EmbeddingCreateParams
from .completion_create_params import CompletionCreateParams as CompletionCreateParams
from .audio_speech_stream_chunk import AudioSpeechStreamChunk as AudioSpeechStreamChunk
from .fine_tuning_delete_params import FineTuningDeleteParams as FineTuningDeleteParams
from .fine_tuning_list_response import FineTuningListResponse as FineTuningListResponse
from .fine_tuning_content_params import FineTuningContentParams as FineTuningContentParams
from .fine_tuning_cancel_response import FineTuningCancelResponse as FineTuningCancelResponse
from .fine_tuning_delete_response import FineTuningDeleteResponse as FineTuningDeleteResponse
from .endpoint_list_avzones_response import EndpointListAvzonesResponse as EndpointListAvzonesResponse
from .code_interpreter_execute_params import CodeInterpreterExecuteParams as CodeInterpreterExecuteParams
from .fine_tuning_list_events_response import FineTuningListEventsResponse as FineTuningListEventsResponse
from .fine_tuning_estimate_price_params import FineTuningEstimatePriceParams as FineTuningEstimatePriceParams
from .fine_tuning_estimate_price_response import FineTuningEstimatePriceResponse as FineTuningEstimatePriceResponse
from .fine_tuning_list_checkpoints_response import (
    FineTuningListCheckpointsResponse as FineTuningListCheckpointsResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .autoscaling import Autoscaling

__all__ = ["DedicatedEndpoint"]


class DedicatedEndpoint(BaseModel):
    """Details about a dedicated endpoint deployment"""

    id: str
    """Unique identifier for the endpoint"""

    autoscaling: Autoscaling
    """Configuration for automatic scaling of the endpoint"""

    created_at: datetime
    """Timestamp when the endpoint was created"""

    display_name: str
    """Human-readable name for the endpoint"""

    hardware: str
    """The hardware configuration used for this endpoint"""

    model: str
    """The model deployed on this endpoint"""

    name: str
    """System name for the endpoint"""

    object: Literal["endpoint"]
    """The type of object"""

    owner: str
    """The owner of this endpoint"""

    state: Literal["PENDING", "STARTING", "STARTED", "STOPPING", "STOPPED", "ERROR"]
    """Current state of the endpoint"""

    type: Literal["dedicated"]
    """The type of endpoint"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TrainingMethodSft"]


class TrainingMethodSft(BaseModel):
    method: Literal["sft"] = "sft"

    train_on_inputs: Union[bool, Literal["auto"]]
    """
    Whether to mask the user messages in conversational data or prompts in
    instruction data.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["EndpointListParams"]


class EndpointListParams(TypedDict, total=False):
    mine: bool
    """If true, return only endpoints owned by the caller"""

    type: Literal["dedicated", "serverless"]
    """Filter endpoints by type"""

    usage_type: Literal["on-demand", "reserved"]
    """Filter endpoints by usage type"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_custom_raw_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ..._streaming import Stream, AsyncStream
from ...types.audio import speech_create_params
from ..._base_client import make_request_options
from ...types.audio_speech_stream_chunk import AudioSpeechStreamChunk

__all__ = ["SpeechResource", "AsyncSpeechResource"]


class SpeechResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return SpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return SpeechResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        input: str,
        model: Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str],
        voice: str,
        language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
        | Omit = omit,
        response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"] | Omit = omit,
        response_format: Literal["mp3", "wav", "raw"] | Omit = omit,
        sample_rate: int | Omit = omit,
        stream: Literal[False] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Generate audio from input text

        Args:
          input: Input text to generate the audio for

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#audio-models)
              The current supported tts models are: - cartesia/sonic - hexgrad/Kokoro-82M -
              canopylabs/orpheus-3b-0.1-ft

          voice: The voice to use for generating the audio. The voices supported are different
              for each model. For eg - for canopylabs/orpheus-3b-0.1-ft, one of the voices
              supported is tara, for hexgrad/Kokoro-82M, one of the voices supported is
              af_alloy and for cartesia/sonic, one of the voices supported is "friendly
              sidekick".

              You can view the voices supported for each model using the /v1/voices endpoint
              sending the model name as the query parameter.
              [View all supported voices here](https://docs.together.ai/docs/text-to-speech#voices-available).

          language: Language of input text.

          response_encoding: Audio encoding of response

          response_format: The format of audio output. Supported formats are mp3, wav, raw if streaming is
              false. If streaming is true, the only supported format is raw.

          sample_rate: Sampling rate to use for the output audio. The default sampling rate for
              canopylabs/orpheus-3b-0.1-ft and hexgrad/Kokoro-82M is 24000 and for
              cartesia/sonic is 44100.

          stream: If true, output is streamed for several characters at a time instead of waiting
              for the full response. The stream terminates with `data: [DONE]`. If false,
              return the encoded audio as octet stream

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        input: str,
        model: Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str],
        stream: Literal[True],
        voice: str,
        language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
        | Omit = omit,
        response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"] | Omit = omit,
        response_format: Literal["mp3", "wav", "raw"] | Omit = omit,
        sample_rate: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[AudioSpeechStreamChunk]:
        """
        Generate audio from input text

        Args:
          input: Input text to generate the audio for

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#audio-models)
              The current supported tts models are: - cartesia/sonic - hexgrad/Kokoro-82M -
              canopylabs/orpheus-3b-0.1-ft

          stream: If true, output is streamed for several characters at a time instead of waiting
              for the full response. The stream terminates with `data: [DONE]`. If false,
              return the encoded audio as octet stream

          voice: The voice to use for generating the audio. The voices supported are different
              for each model. For eg - for canopylabs/orpheus-3b-0.1-ft, one of the voices
              supported is tara, for hexgrad/Kokoro-82M, one of the voices supported is
              af_alloy and for cartesia/sonic, one of the voices supported is "friendly
              sidekick".

              You can view the voices supported for each model using the /v1/voices endpoint
              sending the model name as the query parameter.
              [View all supported voices here](https://docs.together.ai/docs/text-to-speech#voices-available).

          language: Language of input text.

          response_encoding: Audio encoding of response

          response_format: The format of audio output. Supported formats are mp3, wav, raw if streaming is
              false. If streaming is true, the only supported format is raw.

          sample_rate: Sampling rate to use for the output audio. The default sampling rate for
              canopylabs/orpheus-3b-0.1-ft and hexgrad/Kokoro-82M is 24000 and for
              cartesia/sonic is 44100.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        input: str,
        model: Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str],
        stream: bool,
        voice: str,
        language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
        | Omit = omit,
        response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"] | Omit = omit,
        response_format: Literal["mp3", "wav", "raw"] | Omit = omit,
        sample_rate: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse | Stream[AudioSpeechStreamChunk]:
        """
        Generate audio from input text

        Args:
          input: Input text to generate the audio for

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#audio-models)
              The current supported tts models are: - cartesia/sonic - hexgrad/Kokoro-82M -
              canopylabs/orpheus-3b-0.1-ft

          stream: If true, output is streamed for several characters at a time instead of waiting
              for the full response. The stream terminates with `data: [DONE]`. If false,
              return the encoded audio as octet stream

          voice: The voice to use for generating the audio. The voices supported are different
              for each model. For eg - for canopylabs/orpheus-3b-0.1-ft, one of the voices
              supported is tara, for hexgrad/Kokoro-82M, one of the voices supported is
              af_alloy and for cartesia/sonic, one of the voices supported is "friendly
              sidekick".

              You can view the voices supported for each model using the /v1/voices endpoint
              sending the model name as the query parameter.
              [View all supported voices here](https://docs.together.ai/docs/text-to-speech#voices-available).

          language: Language of input text.

          response_encoding: Audio encoding of response

          response_format: The format of audio output. Supported formats are mp3, wav, raw if streaming is
              false. If streaming is true, the only supported format is raw.

          sample_rate: Sampling rate to use for the output audio. The default sampling rate for
              canopylabs/orpheus-3b-0.1-ft and hexgrad/Kokoro-82M is 24000 and for
              cartesia/sonic is 44100.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["input", "model", "voice"], ["input", "model", "stream", "voice"])
    def create(
        self,
        *,
        input: str,
        model: Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str],
        voice: str,
        language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
        | Omit = omit,
        response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"] | Omit = omit,
        response_format: Literal["mp3", "wav", "raw"] | Omit = omit,
        sample_rate: int | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse | Stream[AudioSpeechStreamChunk]:
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._post(
            "/audio/speech",
            body=maybe_transform(
                {
                    "input": input,
                    "model": model,
                    "voice": voice,
                    "language": language,
                    "response_encoding": response_encoding,
                    "response_format": response_format,
                    "sample_rate": sample_rate,
                    "stream": stream,
                },
                speech_create_params.SpeechCreateParamsStreaming
                if stream
                else speech_create_params.SpeechCreateParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
            stream=stream or False,
            stream_cls=Stream[AudioSpeechStreamChunk],
        )


class AsyncSpeechResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncSpeechResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        input: str,
        model: Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str],
        voice: str,
        language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
        | Omit = omit,
        response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"] | Omit = omit,
        response_format: Literal["mp3", "wav", "raw"] | Omit = omit,
        sample_rate: int | Omit = omit,
        stream: Literal[False] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Generate audio from input text

        Args:
          input: Input text to generate the audio for

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#audio-models)
              The current supported tts models are: - cartesia/sonic - hexgrad/Kokoro-82M -
              canopylabs/orpheus-3b-0.1-ft

          voice: The voice to use for generating the audio. The voices supported are different
              for each model. For eg - for canopylabs/orpheus-3b-0.1-ft, one of the voices
              supported is tara, for hexgrad/Kokoro-82M, one of the voices supported is
              af_alloy and for cartesia/sonic, one of the voices supported is "friendly
              sidekick".

              You can view the voices supported for each model using the /v1/voices endpoint
              sending the model name as the query parameter.
              [View all supported voices here](https://docs.together.ai/docs/text-to-speech#voices-available).

          language: Language of input text.

          response_encoding: Audio encoding of response

          response_format: The format of audio output. Supported formats are mp3, wav, raw if streaming is
              false. If streaming is true, the only supported format is raw.

          sample_rate: Sampling rate to use for the output audio. The default sampling rate for
              canopylabs/orpheus-3b-0.1-ft and hexgrad/Kokoro-82M is 24000 and for
              cartesia/sonic is 44100.

          stream: If true, output is streamed for several characters at a time instead of waiting
              for the full response. The stream terminates with `data: [DONE]`. If false,
              return the encoded audio as octet stream

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        input: str,
        model: Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str],
        stream: Literal[True],
        voice: str,
        language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
        | Omit = omit,
        response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"] | Omit = omit,
        response_format: Literal["mp3", "wav", "raw"] | Omit = omit,
        sample_rate: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[AudioSpeechStreamChunk]:
        """
        Generate audio from input text

        Args:
          input: Input text to generate the audio for

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#audio-models)
              The current supported tts models are: - cartesia/sonic - hexgrad/Kokoro-82M -
              canopylabs/orpheus-3b-0.1-ft

          stream: If true, output is streamed for several characters at a time instead of waiting
              for the full response. The stream terminates with `data: [DONE]`. If false,
              return the encoded audio as octet stream

          voice: The voice to use for generating the audio. The voices supported are different
              for each model. For eg - for canopylabs/orpheus-3b-0.1-ft, one of the voices
              supported is tara, for hexgrad/Kokoro-82M, one of the voices supported is
              af_alloy and for cartesia/sonic, one of the voices supported is "friendly
              sidekick".

              You can view the voices supported for each model using the /v1/voices endpoint
              sending the model name as the query parameter.
              [View all supported voices here](https://docs.together.ai/docs/text-to-speech#voices-available).

          language: Language of input text.

          response_encoding: Audio encoding of response

          response_format: The format of audio output. Supported formats are mp3, wav, raw if streaming is
              false. If streaming is true, the only supported format is raw.

          sample_rate: Sampling rate to use for the output audio. The default sampling rate for
              canopylabs/orpheus-3b-0.1-ft and hexgrad/Kokoro-82M is 24000 and for
              cartesia/sonic is 44100.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        input: str,
        model: Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str],
        stream: bool,
        voice: str,
        language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
        | Omit = omit,
        response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"] | Omit = omit,
        response_format: Literal["mp3", "wav", "raw"] | Omit = omit,
        sample_rate: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse | AsyncStream[AudioSpeechStreamChunk]:
        """
        Generate audio from input text

        Args:
          input: Input text to generate the audio for

          model: The name of the model to query.

              [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#audio-models)
              The current supported tts models are: - cartesia/sonic - hexgrad/Kokoro-82M -
              canopylabs/orpheus-3b-0.1-ft

          stream: If true, output is streamed for several characters at a time instead of waiting
              for the full response. The stream terminates with `data: [DONE]`. If false,
              return the encoded audio as octet stream

          voice: The voice to use for generating the audio. The voices supported are different
              for each model. For eg - for canopylabs/orpheus-3b-0.1-ft, one of the voices
              supported is tara, for hexgrad/Kokoro-82M, one of the voices supported is
              af_alloy and for cartesia/sonic, one of the voices supported is "friendly
              sidekick".

              You can view the voices supported for each model using the /v1/voices endpoint
              sending the model name as the query parameter.
              [View all supported voices here](https://docs.together.ai/docs/text-to-speech#voices-available).

          language: Language of input text.

          response_encoding: Audio encoding of response

          response_format: The format of audio output. Supported formats are mp3, wav, raw if streaming is
              false. If streaming is true, the only supported format is raw.

          sample_rate: Sampling rate to use for the output audio. The default sampling rate for
              canopylabs/orpheus-3b-0.1-ft and hexgrad/Kokoro-82M is 24000 and for
              cartesia/sonic is 44100.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["input", "model", "voice"], ["input", "model", "stream", "voice"])
    async def create(
        self,
        *,
        input: str,
        model: Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str],
        voice: str,
        language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
        | Omit = omit,
        response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"] | Omit = omit,
        response_format: Literal["mp3", "wav", "raw"] | Omit = omit,
        sample_rate: int | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse | AsyncStream[AudioSpeechStreamChunk]:
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._post(
            "/audio/speech",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "model": model,
                    "voice": voice,
                    "language": language,
                    "response_encoding": response_encoding,
                    "response_format": response_format,
                    "sample_rate": sample_rate,
                    "stream": stream,
                },
                speech_create_params.SpeechCreateParamsStreaming
                if stream
                else speech_create_params.SpeechCreateParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
            stream=stream or False,
            stream_cls=AsyncStream[AudioSpeechStreamChunk],
        )


class SpeechResourceWithRawResponse:
    def __init__(self, speech: SpeechResource) -> None:
        self._speech = speech

        self.create = to_custom_raw_response_wrapper(
            speech.create,
            BinaryAPIResponse,
        )


class AsyncSpeechResourceWithRawResponse:
    def __init__(self, speech: AsyncSpeechResource) -> None:
        self._speech = speech

        self.create = async_to_custom_raw_response_wrapper(
            speech.create,
            AsyncBinaryAPIResponse,
        )


class SpeechResourceWithStreamingResponse:
    def __init__(self, speech: SpeechResource) -> None:
        self._speech = speech

        self.create = to_custom_streamed_response_wrapper(
            speech.create,
            StreamedBinaryAPIResponse,
        )


class AsyncSpeechResourceWithStreamingResponse:
    def __init__(self, speech: AsyncSpeechResource) -> None:
        self._speech = speech

        self.create = async_to_custom_streamed_response_wrapper(
            speech.create,
            AsyncStreamedBinaryAPIResponse,
        )



import json

import pytest
from httpx import (
    URL,
    Request,
    Response,
)
from pytest_mock import MockerFixture

from together import Together
from together._compat import parse_obj
from together.types.execute_response import (
    SuccessfulExecution,
    SuccessfulExecutionData,
    SuccessfulExecutionDataOutputError,
    SuccessfulExecutionDataOutputStreamOutput,
    SuccessfulExecutionDataOutputDisplayorExecuteOutput,
    SuccessfulExecutionDataOutputDisplayorExecuteOutputData,
)
from together.resources.code_interpreter import CodeInterpreterResource
from together.types.code_interpreter_execute_params import (
    File,
)


def test_interpreter_output_validation():
    # Test stdout output
    stdout = SuccessfulExecutionDataOutputStreamOutput(type="stdout", data="Hello, world!")
    assert stdout.type == "stdout"
    assert stdout.data == "Hello, world!"

    # Test stderr output
    stderr = SuccessfulExecutionDataOutputStreamOutput(type="stderr", data="Warning message")
    assert stderr.type == "stderr"
    assert stderr.data == "Warning message"

    # Test error output
    error = SuccessfulExecutionDataOutputError(type="error", data="Error occurred")
    assert error.type == "error"
    assert error.data == "Error occurred"

    # Test display_data output with dict data
    display_data = SuccessfulExecutionDataOutputDisplayorExecuteOutput(
        type="display_data",
        data=parse_obj(
            SuccessfulExecutionDataOutputDisplayorExecuteOutputData,
            {
                "text/plain": "Hello",
                "text/html": "<p>Hello</p>",
            },
        ),
    )
    assert display_data.type == "display_data"
    assert isinstance(display_data.data, SuccessfulExecutionDataOutputDisplayorExecuteOutputData)
    assert display_data.data.text_plain == "Hello"
    assert display_data.data.text_html == "<p>Hello</p>"

    # Test execute_result output
    execute_result = SuccessfulExecutionDataOutputDisplayorExecuteOutput(
        type="execute_result",
        data=parse_obj(
            SuccessfulExecutionDataOutputDisplayorExecuteOutputData,
            {
                "text/plain": "42",
            },
        ),
    )
    assert execute_result.type == "execute_result"
    assert execute_result.data.text_plain == "42"


def test_execute_response_validation():
    # Test valid response
    outputs = [
        SuccessfulExecutionDataOutputStreamOutput(type="stdout", data="Hello"),
        SuccessfulExecutionDataOutputStreamOutput(type="stderr", data="Warning"),
    ]
    response = SuccessfulExecution(
        data=SuccessfulExecutionData(
            session_id="test_session",
            status="success",
            outputs=outputs,  # type: ignore
        )
    )
    assert response.data.session_id == "test_session"
    assert response.data.status == "success"
    assert len(response.data.outputs) == 2
    assert response.data.outputs[0].type == "stdout"
    assert response.data.outputs[1].type == "stderr"


def test_code_interpreter_run(mocker: MockerFixture):
    # Mock the API requestor
    response_data = {
        "data": {
            "session_id": "test_session",
            "status": "success",
            "outputs": [{"type": "stdout", "data": "Hello, world!"}],
        }
    }
    mock_headers = {
        "cf-ray": "test-ray-id-files",
        "x-ratelimit-remaining": "98",
        "x-hostname": "test-host",
        "x-total-time": "42.0",
    }
    mock_request = mocker.MagicMock()
    mock_request.headers = {}
    mock_response = Response(status_code=200, json=response_data, headers=mock_headers, request=mock_request)
    mock_requestor = mocker.MagicMock()
    mock_requestor.return_value = mock_response

    # Mock the post method directly on the client
    client = Together(api_key="fake_api_key")
    mocker.patch.object(client._client, "send", mock_requestor)
    interpreter = CodeInterpreterResource(client)

    # Test run method
    response = interpreter.execute(
        code='print("Hello, world!")',
        language="python",
        session_id="test_session",
    )

    # Verify the response
    assert isinstance(response, SuccessfulExecution)
    assert response.data is not None
    assert response.data.session_id == "test_session"
    assert response.data.status == "success"
    assert len(response.data.outputs) == 1
    assert response.data.outputs[0].type == "stdout"
    assert response.data.outputs[0].data == "Hello, world!"

    # Verify API request
    mock_requestor.assert_called_once_with(
        mocker.ANY,
        stream=False,
    )
    request = mock_requestor.call_args[0][0]
    assert isinstance(request, Request)
    assert request.method == "POST"
    assert request.url == URL("https://api.together.xyz/v1/tci/execute")
    assert json.loads(request.read().decode()) == {
        "code": 'print("Hello, world!")',
        "language": "python",
        "session_id": "test_session",
    }


def test_code_interpreter_run_without_session(mocker: MockerFixture):
    # Mock the API requestor
    response_data = {  # type: ignore
        "data": {
            "session_id": "new_session",
            "status": "success",
            "outputs": [],
        }
    }
    mock_headers = {
        "cf-ray": "test-ray-id-files",
        "x-ratelimit-remaining": "98",
        "x-hostname": "test-host",
        "x-total-time": "42.0",
    }
    mock_request = mocker.MagicMock()
    mock_request.headers = {}
    mock_response = Response(status_code=200, json=response_data, headers=mock_headers, request=mock_request)
    mock_requestor = mocker.MagicMock()
    mock_requestor.return_value = mock_response

    # Mock the post method directly on the client
    client = Together(api_key="fake_api_key")
    mocker.patch.object(client._client, "send", mock_requestor)
    interpreter = CodeInterpreterResource(client)

    # Test run method without session_id
    response = interpreter.execute(
        code="x = 1",
        language="python",
    )

    # Verify the response
    assert isinstance(response, SuccessfulExecution)
    assert response.data is not None
    assert response.data.session_id == "new_session"


def test_code_interpreter_error_handling(mocker: MockerFixture):
    # Mock the API requestor to simulate an error
    response_data = {
        "data": {
            "session_id": "test_session",
            "status": "error",
            "outputs": [{"type": "error", "data": "Division by zero"}],
        }
    }
    mock_headers = {
        "cf-ray": "test-ray-id-files",
        "x-ratelimit-remaining": "98",
        "x-hostname": "test-host",
        "x-total-time": "42.0",
    }
    mock_request = mocker.MagicMock()
    mock_request.headers = {}
    mock_response = Response(status_code=200, json=response_data, headers=mock_headers, request=mock_request)
    mock_requestor = mocker.MagicMock()
    mock_requestor.return_value = mock_response

    # Mock the post method directly on the client
    client = Together(api_key="fake_api_key")
    mocker.patch.object(client._client, "send", mock_requestor)
    interpreter = CodeInterpreterResource(client)

    # Test run method with code that would cause an error
    response = interpreter.execute(
        code="1/0",  # This will cause a division by zero error
        language="python",
        session_id="test_session",
    )

    # Verify the error response
    assert isinstance(response, SuccessfulExecution)
    assert response.data is not None
    assert response.data.status == "error"
    assert len(response.data.outputs) == 1
    assert response.data.outputs[0].type == "error"
    assert "Division by zero" in response.data.outputs[0].data


def test_code_interpreter_multiple_outputs(mocker: MockerFixture):
    # Mock the API requestor
    response_data = {
        "data": {
            "session_id": "test_session",
            "status": "success",
            "outputs": [
                {"type": "stdout", "data": "First line"},
                {"type": "stderr", "data": "Warning message"},
                {"type": "execute_result", "data": "42"},
            ],
        }
    }
    mock_headers = {
        "cf-ray": "test-ray-id-files",
        "x-ratelimit-remaining": "98",
        "x-hostname": "test-host",
        "x-total-time": "42.0",
    }
    mock_request = mocker.MagicMock()
    mock_request.headers = {}
    mock_response = Response(status_code=200, json=response_data, headers=mock_headers, request=mock_request)
    mock_requestor = mocker.MagicMock()
    mock_requestor.return_value = mock_response

    # Mock the post method directly on the client
    client = Together(api_key="fake_api_key")
    mocker.patch.object(client._client, "send", mock_requestor)
    interpreter = CodeInterpreterResource(client)

    # Test run method with code that produces multiple outputs
    response = interpreter.execute(
        code='print("First line")\nimport sys\nsys.stderr.write("Warning message")\n42',
        language="python",
        session_id="test_session",
    )

    # Verify the response with multiple outputs
    assert isinstance(response, SuccessfulExecution)
    assert response.data is not None
    assert response.data.status == "success"
    assert len(response.data.outputs) == 3
    assert response.data.outputs[0].type == "stdout"
    assert response.data.outputs[1].type == "stderr"
    assert response.data.outputs[2].type == "execute_result"


def test_code_interpreter_session_management(mocker: MockerFixture):
    # First response - create new session
    response_data1 = {
        "data": {
            "session_id": "new_session",
            "status": "success",
            "outputs": [{"type": "stdout", "data": "First execution"}],
        }
    }

    # Second response - use existing session
    response_data2 = {
        "data": {
            "session_id": "new_session",
            "status": "success",
            "outputs": [{"type": "stdout", "data": "Second execution"}],
        }
    }

    mock_headers = {
        "cf-ray": "test-ray-id-files",
        "x-ratelimit-remaining": "98",
        "x-hostname": "test-host",
        "x-total-time": "42.0",
    }
    mock_request = mocker.MagicMock()
    mock_request.headers = {}
    mock_response1 = Response(status_code=200, json=response_data1, headers=mock_headers, request=mock_request)
    mock_response2 = Response(status_code=200, json=response_data2, headers=mock_headers, request=mock_request)
    # Mock the post method directly on the client
    client = Together(api_key="fake_api_key")
    mock_requestor = mocker.MagicMock()
    mock_requestor.side_effect = [
        mock_response1,
        mock_response2,
    ]
    # The mock_requestor becomes the mock for client._client.send
    mocker.patch.object(client._client, "send", mock_requestor)
    # Now mock_requestor can be used to track calls

    interpreter = CodeInterpreterResource(client)

    # First execution - no session ID
    response1 = interpreter.execute(
        code='print("First execution")',
        language="python",
    )

    # Second execution - using session ID from first execution
    assert response1.data is not None
    response2 = interpreter.execute(
        code='print("Second execution")',
        language="python",
        session_id=response1.data.session_id,
    )

    # Verify both responses
    assert response1.data is not None
    assert response2.data is not None
    assert response1.data.session_id == "new_session"
    assert response2.data.session_id == "new_session"
    assert len(response1.data.outputs) == 1
    assert len(response2.data.outputs) == 1
    assert response1.data.outputs[0].data == "First execution"
    assert response2.data.outputs[0].data == "Second execution"

    # Verify API calls
    assert mock_requestor.call_count == 2
    calls = mock_requestor.call_args_list

    # First call should not have session_id
    request1 = calls[0][0][0]
    assert isinstance(request1, Request)
    assert "session_id" not in json.loads(request1.read().decode())

    # Second call should have session_id
    request2 = calls[1][0][0]
    assert isinstance(request2, Request)
    assert json.loads(request2.read().decode())["session_id"] == "new_session"


def test_code_interpreter_run_with_files(mocker: MockerFixture):
    mock_requestor = mocker.MagicMock()
    response_data = {
        "data": {
            "session_id": "test_session_files",
            "status": "success",
            "outputs": [{"type": "stdout", "data": "File content read"}],
        }
    }
    mock_headers = {
        "cf-ray": "test-ray-id-files",
        "x-ratelimit-remaining": "98",
        "x-hostname": "test-host",
        "x-total-time": "42.0",
    }
    mock_request = mocker.MagicMock()
    mock_request.headers = {}
    mock_response = Response(status_code=200, json=response_data, headers=mock_headers, request=mock_request)
    mock_requestor = mocker.MagicMock()
    mock_requestor.return_value = mock_response

    # Mock the post method directly on the client
    client = Together(api_key="fake_api_key")
    mocker.patch.object(client._client, "send", mock_requestor)
    interpreter = CodeInterpreterResource(client)

    # Define files
    files_to_upload = [
        File({"name": "test.txt", "encoding": "string", "content": "Hello from file!"}),
        File({"name": "image.png", "encoding": "base64", "content": "aW1hZ2UgZGF0YQ=="}),
    ]

    # Test run method with files (passing list of dicts)
    response = interpreter.execute(
        code='with open("test.txt") as f: print(f.read())',
        language="python",
        files=files_to_upload,  # Pass the list of dictionaries directly
    )

    # Verify the response
    assert isinstance(response, SuccessfulExecution)
    assert response.data is not None
    assert response.data.session_id == "test_session_files"
    assert response.data.status == "success"
    assert len(response.data.outputs) == 1
    assert response.data.outputs[0].type == "stdout"

    # Verify API request includes files (expected_files_payload remains the same)
    mock_requestor.assert_called_once_with(
        mocker.ANY,
        stream=False,
    )
    request = mock_requestor.call_args[0][0]
    assert isinstance(request, Request)
    assert request.method == "POST"
    assert request.url == URL("https://api.together.xyz/v1/tci/execute")
    expected_files_payload = [
        {"name": "test.txt", "encoding": "string", "content": "Hello from file!"},
        {"name": "image.png", "encoding": "base64", "content": "aW1hZ2UgZGF0YQ=="},
    ]
    assert json.loads(request.read().decode()) == {
        "code": 'with open("test.txt") as f: print(f.read())',
        "language": "python",
        "files": expected_files_payload,
    }


@pytest.mark.skip(
    "Skipping tests around raising an error on a bad file dict structure.  This is handled with pyright type hinting."
)
def test_code_interpreter_run_with_invalid_file_dict_structure(mocker: MockerFixture):
    """Test that run raises ValueError for missing keys in file dict."""
    client = mocker.MagicMock()
    interpreter = CodeInterpreterResource(client)

    with pytest.raises(ValueError, match="Invalid file input format"):
        invalid_files = [
            File({"name": "test.txt", "content": "Missing encoding"})  # type: ignore Missing 'encoding'
        ]

        interpreter.execute(
            code="print('test')",
            language="python",
            files=invalid_files,
        )


@pytest.mark.skip(
    "Skipping tests around raising an error on a bad file dict structure.  This is handled with pyright type hinting."
)
def test_code_interpreter_run_with_invalid_file_dict_encoding(mocker: MockerFixture):
    """Test that run raises ValueError for invalid encoding value."""
    client = mocker.MagicMock()
    interpreter = CodeInterpreterResource(client)

    with pytest.raises(ValueError, match="Invalid file input format"):
        invalid_files = [
            File(
                {
                    "name": "test.txt",
                    "encoding": "utf-8",  # type: ignore Invalid 'encoding' value
                    "content": "Invalid encoding",
                }
            )
        ]

        interpreter.execute(
            code="print('test')",
            language="python",
            files=invalid_files,
        )


def test_code_interpreter_run_with_invalid_file_list_item(mocker: MockerFixture):
    """Test that run raises ValueError for non-dict item in files list."""
    client = mocker.MagicMock()
    interpreter = CodeInterpreterResource(client)

    with pytest.raises(
        ValueError,
        match="dictionary update sequence element #0 has length 1; 2 is required",
    ):
        invalid_files = [
            File({"name": "good.txt", "encoding": "string", "content": "Good"}),
            File("not a dictionary"),  # type: ignore Invalid item type
        ]

        interpreter.execute(
            code="print('test')",
            language="python",
            files=invalid_files,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .finetune_event_type import FinetuneEventType

__all__ = ["FinetuneEvent"]


class FinetuneEvent(BaseModel):
    checkpoint_path: str

    created_at: str

    hash: str

    message: str

    x_model_path: str = FieldInfo(alias="model_path")

    object: Literal["fine-tune-event"]

    param_count: int

    step: int

    token_count: int

    total_steps: int

    training_offset: int

    type: FinetuneEventType

    wandb_url: str

    level: Optional[Literal["info", "warning", "error", "legacy_info", "legacy_iwarning", "legacy_ierror"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["ChatCompletionUsage"]


class ChatCompletionUsage(BaseModel):
    completion_tokens: int

    prompt_tokens: int

    total_tokens: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["EvalListParams"]


class EvalListParams(TypedDict, total=False):
    limit: int

    status: str

    user_id: Annotated[str, PropertyInfo(alias="userId")]
    """Admin users can specify a user ID to filter jobs.

    Pass empty string to get all jobs.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .evaluation_job import EvaluationJob

__all__ = ["EvalListResponse"]

EvalListResponse: TypeAlias = List[EvaluationJob]



import os

import pytest

from together import Together
from together.types.completion_chunk import Choice, ChoiceDelta, CompletionChunk, ChatCompletionUsage

from .generate_hyperparameters import (
    random_top_k,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_top_p,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_max_tokens,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_temperature,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_repetition_penalty,  # noqa: F401 # pyright: ignore[reportUnusedImport]
)


class TestTogetherCompletionStream:
    @pytest.fixture
    def sync_together_client(self) -> Together:
        """
        Initialize object with mocked API key
        """
        TOGETHER_API_KEY = os.getenv("TOGETHER_API_KEY")
        return Together(api_key=TOGETHER_API_KEY)

    def test_create(
        self,
        sync_together_client: Together,
        random_max_tokens: int,
        random_temperature: float,
        random_top_p: float,
        random_top_k: int,
        random_repetition_penalty: float,
    ) -> None:
        prompt = "The space robots have"
        model = "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo"
        stop = ["</s>"]

        # max_tokens should be a reasonable number for this test
        assert 0 < random_max_tokens < 1024

        assert 0 <= random_temperature <= 2

        assert 0 <= random_top_p <= 1

        assert 1 <= random_top_k <= 100

        assert 0 <= random_repetition_penalty <= 2

        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=stop,
            max_tokens=random_max_tokens,
            temperature=random_temperature,
            top_p=random_top_p,
            top_k=random_top_k,
            repetition_penalty=random_repetition_penalty,
            stream=True,
        )

        usage = None

        for chunk in response:
            assert isinstance(chunk, CompletionChunk)
            assert isinstance(chunk.id, str)
            assert isinstance(chunk.created, int)
            assert chunk.object == "completion.chunk"
            assert isinstance(chunk.choices[0], Choice)
            assert isinstance(chunk.choices[0].index, int)
            assert isinstance(chunk.choices[0].delta, ChoiceDelta)
            assert isinstance(chunk.choices[0].delta.content, str)

            usage = chunk.usage

        assert isinstance(usage, ChatCompletionUsage)
        assert isinstance(usage.prompt_tokens, int)
        assert isinstance(usage.completion_tokens, int)
        assert isinstance(usage.total_tokens, int)
        assert usage.prompt_tokens + usage.completion_tokens == usage.total_tokens

    def test_prompt(self):
        pass

    def test_no_prompt(self):
        pass

    def test_model(self):
        pass

    def test_no_model(self):
        pass

    def test_max_tokens(self):
        pass

    def test_no_max_tokens(self):
        pass

    def test_high_max_tokens(self):
        pass

    def test_stop(self):
        pass

    def test_no_stop(self):
        pass

    def test_temperature(self):
        pass

    def test_top_p(self):
        pass

    def test_top_k(self):
        pass

    def test_repetition_penalty(self):
        pass

    def test_echo(self):
        pass

    def test_n(self):
        pass

    def test_safety_model(self):
        pass



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from .autoscaling_param import AutoscalingParam

__all__ = ["EndpointCreateParams"]


class EndpointCreateParams(TypedDict, total=False):
    autoscaling: Required[AutoscalingParam]
    """Configuration for automatic scaling of the endpoint"""

    hardware: Required[str]
    """The hardware configuration to use for this endpoint"""

    model: Required[str]
    """The model to deploy on this endpoint"""

    availability_zone: str
    """Create the endpoint in a specified availability zone (e.g., us-central-4b)"""

    disable_prompt_cache: bool
    """Whether to disable the prompt cache for this endpoint"""

    disable_speculative_decoding: bool
    """Whether to disable speculative decoding for this endpoint"""

    display_name: str
    """A human-readable name for the endpoint"""

    inactive_timeout: Optional[int]
    """
    The number of minutes of inactivity after which the endpoint will be
    automatically stopped. Set to null, omit or set to 0 to disable automatic
    timeout.
    """

    state: Literal["STARTED", "STOPPED"]
    """The desired state of the endpoint"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["VideoCreateParams", "FrameImage"]


class VideoCreateParams(TypedDict, total=False):
    model: Required[str]
    """The model to be used for the video creation request."""

    fps: int
    """Frames per second. Defaults to 24."""

    frame_images: Iterable[FrameImage]
    """Array of images to guide video generation, similar to keyframes."""

    guidance_scale: int
    """Controls how closely the video generation follows your prompt.

    Higher values make the model adhere more strictly to your text description,
    while lower values allow more creative freedom. guidence_scale affects both
    visual content and temporal consistency.Recommended range is 6.0-10.0 for most
    video models. Values above 12 may cause over-guidance artifacts or unnatural
    motion patterns.
    """

    height: int

    negative_prompt: str
    """Similar to prompt, but specifies what to avoid instead of what to include"""

    output_format: Literal["MP4", "WEBM"]
    """Specifies the format of the output video. Defaults to MP4."""

    output_quality: int
    """Compression quality. Defaults to 20."""

    prompt: str
    """Text prompt that describes the video to generate."""

    reference_images: SequenceNotStr[str]
    """
    Unlike frame_images which constrain specific timeline positions, reference
    images guide the general appearance that should appear consistently across the
    video.
    """

    seconds: str
    """Clip duration in seconds."""

    seed: int
    """Seed to use in initializing the video generation.

    Using the same seed allows deterministic video generation. If not provided a
    random seed is generated for each request.
    """

    steps: int
    """The number of denoising steps the model performs during video generation.

    More steps typically result in higher quality output but require longer
    processing time.
    """

    width: int


class FrameImage(TypedDict, total=False):
    input_image: Required[str]
    """URL path to hosted image that is used for a frame"""

    frame: Union[float, Literal["first", "last"]]
    """Optional param to specify where to insert the frame.

    If this is omitted, the following heuristics are applied:

    - frame_images size is one, frame is first.
    - If size is two, frames are first and last.
    - If size is larger, frames are first, last and evenly spaced between.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ModelUploadParams"]


class ModelUploadParams(TypedDict, total=False):
    model_name: Required[str]
    """The name to give to your uploaded model"""

    model_source: Required[str]
    """The source location of the model (Hugging Face repo or S3 path)"""

    base_model: str
    """
    The base model to use for an adapter if setting it to run against a serverless
    pool. Only used for model_type `adapter`.
    """

    description: str
    """A description of your model"""

    hf_token: str
    """Hugging Face token (if uploading from Hugging Face)"""

    lora_model: str
    """
    The lora pool to use for an adapter if setting it to run against, say, a
    dedicated pool. Only used for model_type `adapter`.
    """

    model_type: Literal["model", "adapter"]
    """Whether the model is a full model or an adapter"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "TranslationCreateResponse",
    "AudioTranslationJsonResponse",
    "AudioTranslationVerboseJsonResponse",
    "AudioTranslationVerboseJsonResponseSegment",
    "AudioTranslationVerboseJsonResponseWord",
]


class AudioTranslationJsonResponse(BaseModel):
    text: str
    """The translated text"""


class AudioTranslationVerboseJsonResponseSegment(BaseModel):
    id: int
    """Unique identifier for the segment"""

    end: float
    """End time of the segment in seconds"""

    start: float
    """Start time of the segment in seconds"""

    text: str
    """The text content of the segment"""


class AudioTranslationVerboseJsonResponseWord(BaseModel):
    end: float
    """End time of the word in seconds"""

    start: float
    """Start time of the word in seconds"""

    word: str
    """The word"""

    speaker_id: Optional[str] = None
    """The speaker id for the word (only when diarize is enabled)"""


class AudioTranslationVerboseJsonResponse(BaseModel):
    duration: float
    """The duration of the audio in seconds"""

    language: str
    """The target language of the translation"""

    segments: List[AudioTranslationVerboseJsonResponseSegment]
    """Array of translation segments"""

    task: Literal["transcribe", "translate"]
    """The task performed"""

    text: str
    """The translated text"""

    words: Optional[List[AudioTranslationVerboseJsonResponseWord]] = None
    """Array of translation words (only when timestamp_granularities includes 'word')"""


TranslationCreateResponse: TypeAlias = Union[AudioTranslationJsonResponse, AudioTranslationVerboseJsonResponse]



from __future__ import annotations

import sys
import json
from typing import Any, Dict, Literal, TypeVar, Callable, cast
from functools import wraps

import click

from together import Together, omit
from together.types import DedicatedEndpoint
from together._exceptions import APIError
from together.lib.utils.serializer import datetime_serializer
from together.types.endpoint_list_response import Data as DedicatedEndpointListItem


def print_endpoint(endpoint: DedicatedEndpoint | DedicatedEndpointListItem) -> None:
    """Print endpoint details in a Docker-like format or JSON."""

    # Print header info
    click.echo(f"ID:\t\t{endpoint.id}")
    click.echo(f"Name:\t\t{endpoint.name}")

    if isinstance(endpoint, DedicatedEndpoint):
        click.echo(f"Display Name:\t{endpoint.display_name}")
        click.echo(f"Hardware:\t{endpoint.hardware}")
        click.echo(f"Autoscaling:\tMin={endpoint.autoscaling.min_replicas}, Max={endpoint.autoscaling.max_replicas}")

    click.echo(f"Model:\t\t{endpoint.model}")
    click.echo(f"Type:\t\t{endpoint.type}")
    click.echo(f"Owner:\t\t{endpoint.owner}")
    click.echo(f"State:\t\t{endpoint.state}")
    click.echo(f"Created:\t{endpoint.created_at}")


F = TypeVar("F", bound=Callable[..., Any])


def print_api_error(e: APIError) -> None:
    error_details = cast(Dict[str, Any], e.body)["error"]["message"]

    if error_details and ("credentials" in error_details.lower() or "authentication" in error_details.lower()):
        click.echo("Error: Invalid API key or authentication failed", err=True)
    else:
        click.echo(f"Error: {error_details}", err=True)


def handle_api_errors(f: F) -> F:
    """Decorator to handle common API errors in CLI commands."""

    @wraps(f)
    def wrapper(*args: Any, **kwargs: Any) -> Any:
        try:
            return f(*args, **kwargs)
        except APIError as e:
            print_api_error(e)
            sys.exit(1)
        except Exception as e:
            click.echo(f"Error: An unexpected error occurred - {str(e)}", err=True)
            sys.exit(1)

    return wrapper  # type: ignore


@click.group()
@click.pass_context
def endpoints(ctx: click.Context) -> None:
    """Endpoints API commands"""
    pass


@endpoints.command()
@click.option(
    "--model",
    required=True,
    help="The model to deploy (e.g. meta-llama/Llama-4-Scout-17B-16E-Instruct)",
)
@click.option(
    "--min-replicas",
    type=int,
    default=1,
    help="Minimum number of replicas to deploy",
)
@click.option(
    "--max-replicas",
    type=int,
    default=1,
    help="Maximum number of replicas to deploy",
)
@click.option(
    "--gpu",
    type=click.Choice(["h100", "a100", "l40", "l40s", "rtx-6000"]),
    required=True,
    help="GPU type to use for inference",
)
@click.option(
    "--gpu-count",
    type=int,
    default=1,
    help="Number of GPUs to use per replica",
)
@click.option(
    "--display-name",
    help="A human-readable name for the endpoint",
)
@click.option(
    "--no-prompt-cache",
    is_flag=True,
    help="Disable the prompt cache for this endpoint",
)
@click.option(
    "--no-speculative-decoding",
    is_flag=True,
    help="Disable speculative decoding for this endpoint",
)
@click.option(
    "--no-auto-start",
    is_flag=True,
    help="Create the endpoint in STOPPED state instead of auto-starting it",
)
@click.option(
    "--inactive-timeout",
    type=int,
    help="Number of minutes of inactivity after which the endpoint will be automatically stopped. Set to 0 to disable.",
)
@click.option(
    "--availability-zone",
    help="Start endpoint in specified availability zone (e.g., us-central-4b)",
)
@click.option(
    "--wait/--no-wait",
    default=True,
    help="Wait for the endpoint to be ready after creation",
)
@click.pass_obj
@handle_api_errors
def create(
    client: Together,
    model: str,
    min_replicas: int,
    max_replicas: int,
    gpu: str,
    gpu_count: int,
    display_name: str | None,
    no_prompt_cache: bool | None,
    no_speculative_decoding: bool | None,
    no_auto_start: bool,
    inactive_timeout: int | None,
    availability_zone: str | None,
    wait: bool,
) -> None:
    """Create a new dedicated inference endpoint."""
    # Map GPU types to their full hardware ID names
    gpu_map = {
        "h100": "nvidia_h100_80gb_sxm",
        "a100": "nvidia_a100_80gb_pcie" if gpu_count == 1 else "nvidia_a100_80gb_sxm",
        "l40": "nvidia_l40",
        "l40s": "nvidia_l40s",
        "rtx-6000": "nvidia_rtx_6000_ada",
    }

    hardware_id = f"{gpu_count}x_{gpu_map[gpu]}"

    try:
        response = client.endpoints.create(
            model=model,
            hardware=hardware_id,
            autoscaling={
                "min_replicas": min_replicas,
                "max_replicas": max_replicas,
            },
            display_name=display_name or omit,
            disable_prompt_cache=no_prompt_cache or omit,
            disable_speculative_decoding=no_speculative_decoding or omit,
            state="STOPPED" if no_auto_start else "STARTED",
            inactive_timeout=inactive_timeout,
            extra_query={"availability_zone": availability_zone or omit},
        )
    except APIError as e:
        print_api_error(e)
        if "check the hardware api" in str(e.args[0]).lower() or "invalid hardware provided" in str(e.args[0]).lower():
            fetch_and_print_hardware_options(client=client, model=model, print_json=False, available=True)

        sys.exit(1)

    # Print detailed information to stderr
    click.echo("Created dedicated endpoint with:", err=True)
    click.echo(f"  Model: {model}", err=True)
    click.echo(f"  Min replicas: {min_replicas}", err=True)
    click.echo(f"  Max replicas: {max_replicas}", err=True)
    click.echo(f"  Hardware: {hardware_id}", err=True)
    if display_name:
        click.echo(f"  Display name: {display_name}", err=True)
    if no_prompt_cache:
        click.echo("  Prompt cache: disabled", err=True)
    if no_speculative_decoding:
        click.echo("  Speculative decoding: disabled", err=True)
    if no_auto_start:
        click.echo("  Auto-start: disabled", err=True)
    if inactive_timeout is not None:
        click.echo(f"  Inactive timeout: {inactive_timeout} minutes", err=True)
    if availability_zone:
        click.echo(f"  Availability zone: {availability_zone}", err=True)

    click.echo(f"Endpoint created successfully, id: {response.id}", err=True)

    if wait:
        import time

        click.echo("Waiting for endpoint to be ready...", err=True)
        while client.endpoints.retrieve(response.id).state != "STARTED":
            time.sleep(1)
        click.echo("Endpoint ready", err=True)

    # Print only the endpoint ID to stdout
    click.echo(response.id)


@endpoints.command()
@click.argument("endpoint-id", required=True)
@click.option("--json", is_flag=True, help="Print output in JSON format")
@click.pass_obj
@handle_api_errors
def get(client: Together, endpoint_id: str, json: bool) -> None:
    """Get a dedicated inference endpoint."""
    endpoint = client.endpoints.retrieve(endpoint_id)
    if json:
        import json as json_lib

        click.echo(json_lib.dumps(endpoint.model_dump(), indent=2, default=datetime_serializer))
    else:
        print_endpoint(endpoint)


@endpoints.command()
@click.option("--model", help="Filter hardware options by model")
@click.option("--json", is_flag=True, help="Print output in JSON format")
@click.option(
    "--available",
    is_flag=True,
    help="Print only available hardware options (can only be used if model is passed in)",
)
@click.pass_obj
@handle_api_errors
def hardware(client: Together, model: str | None, json: bool, available: bool) -> None:
    """List all available hardware options, optionally filtered by model."""
    fetch_and_print_hardware_options(client, model, json, available)


def fetch_and_print_hardware_options(client: Together, model: str | None, print_json: bool, available: bool) -> None:
    """Print hardware options for a model."""

    message = "Available hardware options:" if available else "All hardware options:"
    click.echo(message, err=True)
    hardware_options = client.hardware.list(model=model or omit)
    # hardware_options = client.endpoints.list_hardware(model)
    if available:
        hardware_options.data = [
            hardware
            for hardware in hardware_options.data
            if hardware.availability is not None and hardware.availability.status == "available"
        ]

    if print_json:
        json_output = [hardware.model_dump() for hardware in hardware_options.data]
        click.echo(json.dumps(json_output, default=datetime_serializer, indent=2))
    else:
        for hardware in hardware_options.data:
            click.echo(f"  {hardware.id}", err=True)


@endpoints.command()
@click.argument("endpoint-id", required=True)
@click.option("--wait/--no-wait", default=True, help="Wait for the endpoint to stop")
@click.pass_obj
@handle_api_errors
def stop(client: Together, endpoint_id: str, wait: bool) -> None:
    """Stop a dedicated inference endpoint."""
    client.endpoints.update(endpoint_id, state="STOPPED")
    click.echo("Successfully marked endpoint as stopping", err=True)

    if wait:
        import time

        click.echo("Waiting for endpoint to stop...", err=True)
        while client.endpoints.retrieve(endpoint_id).state != "STOPPED":
            time.sleep(1)
        click.echo("Endpoint stopped", err=True)

    click.echo(endpoint_id)


@endpoints.command()
@click.argument("endpoint-id", required=True)
@click.option("--wait/--no-wait", default=True, help="Wait for the endpoint to start")
@click.pass_obj
@handle_api_errors
def start(client: Together, endpoint_id: str, wait: bool) -> None:
    """Start a dedicated inference endpoint."""
    client.endpoints.update(endpoint_id, state="STARTED")
    click.echo("Successfully marked endpoint as starting", err=True)

    if wait:
        import time

        click.echo("Waiting for endpoint to start...", err=True)
        while client.endpoints.retrieve(endpoint_id).state != "STARTED":
            time.sleep(1)
        click.echo("Endpoint started", err=True)

    click.echo(endpoint_id)


@endpoints.command()
@click.argument("endpoint-id", required=True)
@click.pass_obj
@handle_api_errors
def delete(client: Together, endpoint_id: str) -> None:
    """Delete a dedicated inference endpoint."""
    client.endpoints.delete(endpoint_id)
    click.echo("Successfully deleted endpoint", err=True)
    click.echo(endpoint_id)


@endpoints.command()
@click.option("--json", is_flag=True, help="Print output in JSON format")
@click.option(
    "--type",
    type=click.Choice(["dedicated", "serverless"]),
    help="Filter by endpoint type",
)
@click.option(
    "--mine",
    type=click.BOOL,
    default=None,
    help="true (only mine), default=all",
)
@click.option(
    "--usage-type",
    type=click.Choice(["on-demand", "reserved"]),
    help="Filter by endpoint usage type",
)
@click.pass_obj
@handle_api_errors
def list(
    client: Together,
    json: bool,
    type: Literal["dedicated", "serverless"] | None,
    usage_type: Literal["on-demand", "reserved"] | None,
    mine: bool | None,
) -> None:
    """List all inference endpoints (includes both dedicated and serverless endpoints)."""
    endpoints = client.endpoints.list(
        type=type or omit,
        usage_type=usage_type or omit,
        mine=mine if mine is not None else omit,
    )

    if not endpoints:
        click.echo("No dedicated endpoints found", err=True)
        return

    click.echo("Endpoints:", err=True)
    if json:
        import json as json_lib

        click.echo(
            json_lib.dumps(
                [endpoint.model_dump() for endpoint in endpoints.data], default=datetime_serializer, indent=2
            )
        )
    else:
        for endpoint in endpoints.data:
            print_endpoint(
                endpoint,
            )
            click.echo()


@endpoints.command()
@click.argument("endpoint-id", required=True)
@click.option(
    "--display-name",
    help="A new human-readable name for the endpoint",
)
@click.option(
    "--min-replicas",
    type=int,
    help="New minimum number of replicas to maintain",
)
@click.option(
    "--max-replicas",
    type=int,
    help="New maximum number of replicas to scale up to",
)
@click.option(
    "--inactive-timeout",
    type=int,
    help="Number of minutes of inactivity after which the endpoint will be automatically stopped. Set to 0 to disable.",
)
@click.pass_obj
@handle_api_errors
def update(
    client: Together,
    endpoint_id: str,
    display_name: str | None,
    min_replicas: int | None,
    max_replicas: int | None,
    inactive_timeout: int | None,
) -> None:
    """Update a dedicated inference endpoint's configuration."""
    if not any([display_name, min_replicas, max_replicas, inactive_timeout]):
        click.echo("Error: At least one update option must be specified", err=True)
        sys.exit(1)

    # Build kwargs for the update
    kwargs: Dict[str, Any] = {}
    if display_name is not None:
        kwargs["display_name"] = display_name

    if min_replicas is not None or max_replicas is not None:
        kwargs["autoscaling"] = {}
        if min_replicas is not None:
            kwargs["autoscaling"]["min_replicas"] = min_replicas
        if max_replicas is not None:
            kwargs["autoscaling"]["max_replicas"] = max_replicas

    if inactive_timeout is not None:
        kwargs["inactive_timeout"] = inactive_timeout

    client.endpoints.update(endpoint_id, **kwargs)

    # Print what was updated
    click.echo("Updated endpoint configuration:", err=True)
    if display_name:
        click.echo(f"  Display name: {display_name}", err=True)
    if min_replicas:
        click.echo(f"  Min replicas: {min_replicas}", err=True)
    if max_replicas:
        click.echo(f"  Max replicas: {max_replicas}", err=True)
    if inactive_timeout is not None:
        click.echo(f"  Inactive timeout: {inactive_timeout} minutes", err=True)

    click.echo("Successfully updated endpoint", err=True)
    click.echo(endpoint_id)


@endpoints.command()
@click.option("--json", is_flag=True, help="Print output in JSON format")
@click.pass_obj
@handle_api_errors
def availability_zones(client: Together, json: bool) -> None:
    """List all availability zones."""
    avzones = client.endpoints.list_avzones()

    if not avzones:
        click.echo("No availability zones found", err=True)
        return

    if json:
        import json as json_lib

        click.echo(json_lib.dumps(avzones.model_dump(), indent=2))
    else:
        click.echo("Available zones:", err=True)
        for availability_zone in sorted(avzones.avzones):
            click.echo(f"  {availability_zone}")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypedDict

__all__ = ["SpeechCreateParamsBase", "SpeechCreateParamsNonStreaming", "SpeechCreateParamsStreaming"]


class SpeechCreateParamsBase(TypedDict, total=False):
    input: Required[str]
    """Input text to generate the audio for"""

    model: Required[Union[Literal["cartesia/sonic", "hexgrad/Kokoro-82M", "canopylabs/orpheus-3b-0.1-ft"], str]]
    """The name of the model to query.

    [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#audio-models)
    The current supported tts models are: - cartesia/sonic - hexgrad/Kokoro-82M -
    canopylabs/orpheus-3b-0.1-ft
    """

    voice: Required[str]
    """The voice to use for generating the audio.

    The voices supported are different for each model. For eg - for
    canopylabs/orpheus-3b-0.1-ft, one of the voices supported is tara, for
    hexgrad/Kokoro-82M, one of the voices supported is af_alloy and for
    cartesia/sonic, one of the voices supported is "friendly sidekick".

    You can view the voices supported for each model using the /v1/voices endpoint
    sending the model name as the query parameter.
    [View all supported voices here](https://docs.together.ai/docs/text-to-speech#voices-available).
    """

    language: Literal["en", "de", "fr", "es", "hi", "it", "ja", "ko", "nl", "pl", "pt", "ru", "sv", "tr", "zh"]
    """Language of input text."""

    response_encoding: Literal["pcm_f32le", "pcm_s16le", "pcm_mulaw", "pcm_alaw"]
    """Audio encoding of response"""

    response_format: Literal["mp3", "wav", "raw"]
    """The format of audio output.

    Supported formats are mp3, wav, raw if streaming is false. If streaming is true,
    the only supported format is raw.
    """

    sample_rate: int
    """Sampling rate to use for the output audio.

    The default sampling rate for canopylabs/orpheus-3b-0.1-ft and
    hexgrad/Kokoro-82M is 24000 and for cartesia/sonic is 44100.
    """


class SpeechCreateParamsNonStreaming(SpeechCreateParamsBase, total=False):
    stream: Literal[False]
    """
    If true, output is streamed for several characters at a time instead of waiting
    for the full response. The stream terminates with `data: [DONE]`. If false,
    return the encoded audio as octet stream
    """


class SpeechCreateParamsStreaming(SpeechCreateParamsBase):
    stream: Required[Literal[True]]
    """
    If true, output is streamed for several characters at a time instead of waiting
    for the full response. The stream terminates with `data: [DONE]`. If false,
    return the encoded audio as octet stream
    """


SpeechCreateParams = Union[SpeechCreateParamsNonStreaming, SpeechCreateParamsStreaming]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from ..log_probs import LogProbs
from ..tool_choice import ToolChoice
from .chat_completion_usage import ChatCompletionUsage
from .chat_completion_warning import ChatCompletionWarning

__all__ = ["ChatCompletion", "Choice", "ChoiceMessage", "ChoiceMessageFunctionCall"]


class ChoiceMessageFunctionCall(BaseModel):
    arguments: str

    name: str


class ChoiceMessage(BaseModel):
    content: Optional[str] = None

    role: Literal["assistant"]

    function_call: Optional[ChoiceMessageFunctionCall] = None

    reasoning: Optional[str] = None

    tool_calls: Optional[List[ToolChoice]] = None


class Choice(BaseModel):
    finish_reason: Optional[Literal["stop", "eos", "length", "tool_calls", "function_call"]] = None

    index: Optional[int] = None

    logprobs: Optional[LogProbs] = None

    message: Optional[ChoiceMessage] = None

    seed: Optional[int] = None

    text: Optional[str] = None


class ChatCompletion(BaseModel):
    id: str

    choices: List[Choice]

    created: int

    model: str

    object: Literal["chat.completion"]

    usage: Optional[ChatCompletionUsage] = None

    warnings: Optional[List[ChatCompletionWarning]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ImageDataB64"]


class ImageDataB64(BaseModel):
    b64_json: str

    index: int

    type: Literal["b64_json"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .chat_completion import ChatCompletion as ChatCompletion
from .chat_completion_chunk import ChatCompletionChunk as ChatCompletionChunk
from .chat_completion_usage import ChatCompletionUsage as ChatCompletionUsage
from .chat_completion_warning import ChatCompletionWarning as ChatCompletionWarning
from .completion_create_params import CompletionCreateParams as CompletionCreateParams
from .chat_completion_structured_message_text_param import (
    ChatCompletionStructuredMessageTextParam as ChatCompletionStructuredMessageTextParam,
)
from .chat_completion_structured_message_image_url_param import (
    ChatCompletionStructuredMessageImageURLParam as ChatCompletionStructuredMessageImageURLParam,
)
from .chat_completion_structured_message_video_url_param import (
    ChatCompletionStructuredMessageVideoURLParam as ChatCompletionStructuredMessageVideoURLParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .audio import (
    AudioResource,
    AsyncAudioResource,
    AudioResourceWithRawResponse,
    AsyncAudioResourceWithRawResponse,
    AudioResourceWithStreamingResponse,
    AsyncAudioResourceWithStreamingResponse,
)
from .evals import (
    EvalsResource,
    AsyncEvalsResource,
    EvalsResourceWithRawResponse,
    AsyncEvalsResourceWithRawResponse,
    EvalsResourceWithStreamingResponse,
    AsyncEvalsResourceWithStreamingResponse,
)
from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .images import (
    ImagesResource,
    AsyncImagesResource,
    ImagesResourceWithRawResponse,
    AsyncImagesResourceWithRawResponse,
    ImagesResourceWithStreamingResponse,
    AsyncImagesResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .rerank import (
    RerankResource,
    AsyncRerankResource,
    RerankResourceWithRawResponse,
    AsyncRerankResourceWithRawResponse,
    RerankResourceWithStreamingResponse,
    AsyncRerankResourceWithStreamingResponse,
)
from .videos import (
    VideosResource,
    AsyncVideosResource,
    VideosResourceWithRawResponse,
    AsyncVideosResourceWithRawResponse,
    VideosResourceWithStreamingResponse,
    AsyncVideosResourceWithStreamingResponse,
)
from .batches import (
    BatchesResource,
    AsyncBatchesResource,
    BatchesResourceWithRawResponse,
    AsyncBatchesResourceWithRawResponse,
    BatchesResourceWithStreamingResponse,
    AsyncBatchesResourceWithStreamingResponse,
)
from .hardware import (
    HardwareResource,
    AsyncHardwareResource,
    HardwareResourceWithRawResponse,
    AsyncHardwareResourceWithRawResponse,
    HardwareResourceWithStreamingResponse,
    AsyncHardwareResourceWithStreamingResponse,
)
from .endpoints import (
    EndpointsResource,
    AsyncEndpointsResource,
    EndpointsResourceWithRawResponse,
    AsyncEndpointsResourceWithRawResponse,
    EndpointsResourceWithStreamingResponse,
    AsyncEndpointsResourceWithStreamingResponse,
)
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from .completions import (
    CompletionsResource,
    AsyncCompletionsResource,
    CompletionsResourceWithRawResponse,
    AsyncCompletionsResourceWithRawResponse,
    CompletionsResourceWithStreamingResponse,
    AsyncCompletionsResourceWithStreamingResponse,
)
from .fine_tuning import (
    FineTuningResource,
    AsyncFineTuningResource,
    FineTuningResourceWithRawResponse,
    AsyncFineTuningResourceWithRawResponse,
    FineTuningResourceWithStreamingResponse,
    AsyncFineTuningResourceWithStreamingResponse,
)
from .code_interpreter import (
    CodeInterpreterResource,
    AsyncCodeInterpreterResource,
    CodeInterpreterResourceWithRawResponse,
    AsyncCodeInterpreterResourceWithRawResponse,
    CodeInterpreterResourceWithStreamingResponse,
    AsyncCodeInterpreterResourceWithStreamingResponse,
)

__all__ = [
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "CompletionsResource",
    "AsyncCompletionsResource",
    "CompletionsResourceWithRawResponse",
    "AsyncCompletionsResourceWithRawResponse",
    "CompletionsResourceWithStreamingResponse",
    "AsyncCompletionsResourceWithStreamingResponse",
    "EmbeddingsResource",
    "AsyncEmbeddingsResource",
    "EmbeddingsResourceWithRawResponse",
    "AsyncEmbeddingsResourceWithRawResponse",
    "EmbeddingsResourceWithStreamingResponse",
    "AsyncEmbeddingsResourceWithStreamingResponse",
    "FilesResource",
    "AsyncFilesResource",
    "FilesResourceWithRawResponse",
    "AsyncFilesResourceWithRawResponse",
    "FilesResourceWithStreamingResponse",
    "AsyncFilesResourceWithStreamingResponse",
    "FineTuningResource",
    "AsyncFineTuningResource",
    "FineTuningResourceWithRawResponse",
    "AsyncFineTuningResourceWithRawResponse",
    "FineTuningResourceWithStreamingResponse",
    "AsyncFineTuningResourceWithStreamingResponse",
    "CodeInterpreterResource",
    "AsyncCodeInterpreterResource",
    "CodeInterpreterResourceWithRawResponse",
    "AsyncCodeInterpreterResourceWithRawResponse",
    "CodeInterpreterResourceWithStreamingResponse",
    "AsyncCodeInterpreterResourceWithStreamingResponse",
    "ImagesResource",
    "AsyncImagesResource",
    "ImagesResourceWithRawResponse",
    "AsyncImagesResourceWithRawResponse",
    "ImagesResourceWithStreamingResponse",
    "AsyncImagesResourceWithStreamingResponse",
    "VideosResource",
    "AsyncVideosResource",
    "VideosResourceWithRawResponse",
    "AsyncVideosResourceWithRawResponse",
    "VideosResourceWithStreamingResponse",
    "AsyncVideosResourceWithStreamingResponse",
    "AudioResource",
    "AsyncAudioResource",
    "AudioResourceWithRawResponse",
    "AsyncAudioResourceWithRawResponse",
    "AudioResourceWithStreamingResponse",
    "AsyncAudioResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "EndpointsResource",
    "AsyncEndpointsResource",
    "EndpointsResourceWithRawResponse",
    "AsyncEndpointsResourceWithRawResponse",
    "EndpointsResourceWithStreamingResponse",
    "AsyncEndpointsResourceWithStreamingResponse",
    "HardwareResource",
    "AsyncHardwareResource",
    "HardwareResourceWithRawResponse",
    "AsyncHardwareResourceWithRawResponse",
    "HardwareResourceWithStreamingResponse",
    "AsyncHardwareResourceWithStreamingResponse",
    "RerankResource",
    "AsyncRerankResource",
    "RerankResourceWithRawResponse",
    "AsyncRerankResourceWithRawResponse",
    "RerankResourceWithStreamingResponse",
    "AsyncRerankResourceWithStreamingResponse",
    "BatchesResource",
    "AsyncBatchesResource",
    "BatchesResourceWithRawResponse",
    "AsyncBatchesResourceWithRawResponse",
    "BatchesResourceWithStreamingResponse",
    "AsyncBatchesResourceWithStreamingResponse",
    "EvalsResource",
    "AsyncEvalsResource",
    "EvalsResourceWithRawResponse",
    "AsyncEvalsResourceWithRawResponse",
    "EvalsResourceWithStreamingResponse",
    "AsyncEvalsResourceWithStreamingResponse",
]
